/*******************************************************************************
 * Copyright 2011 Google Inc. All Rights Reserved.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *******************************************************************************/
package es.upv.client.activity;

import com.google.gwt.activity.shared.AbstractActivity;
import com.google.gwt.event.shared.EventBus;
import com.google.gwt.place.shared.Place;
import com.google.gwt.user.client.ui.AcceptsOneWidget;
import com.google.gwt.user.client.ui.DecoratedPopupPanel;

import es.upv.client.ClientFactory;
import es.upv.client.place.PanelAlumnosPlace;
import es.upv.client.ui.LoginDialog;
import es.upv.client.ui.PanelAlumnoView;
import es.upv.client.ui.UsuarioInvalidoAlert;

/**
 * Activities are started and stopped by an ActivityManager associated with a
 * container Widget.
 */
public class AlumnoActivity extends AbstractActivity implements PanelAlumnoView.Presenter {

	/**
	 * Sample property.
	 */
	private String name;

	private ClientFactory clientFactory;

	public AlumnoActivity(PanelAlumnosPlace place, ClientFactory clientFactory) {
		this.name = place.getName();
		this.clientFactory = clientFactory;
	}

	@Override
	public void start(AcceptsOneWidget containerWidget, EventBus eventBus) {
		PanelAlumnoView view = clientFactory.getPanelAlumnos();
		view.setName(name);
		view.setPresenter(this);
		view.refrescarEvaluaciones();
		containerWidget.setWidget(view.asWidget());
	}

	@Override
	public String mayStop() {
		return null; // return "Please hold on. This activity is stopping.";
	}

	/**
	 * @see PanelAlumnoView.Presenter#goTo(Place)
	 */
	public void goTo(Place place) {
		clientFactory.getPlaceController().goTo(place);
	}

	@Override
	public void mostrarDialogoLogin() {
		LoginDialog ventanaLogin = clientFactory.getLoginDialog();
		ventanaLogin.setListener(this);

		ventanaLogin.usuField.setText("");
		ventanaLogin.passField.setText("");

		ventanaLogin.center();
		ventanaLogin.usuField.getElement().focus();
	}

	@Override
	public void ocultarDialogoLogin() {
		clientFactory.getLoginDialog().hide();
	}

	@Override
	public void muestraErrorCredenciales() {
		LoginDialog ventanaLogin = clientFactory.getLoginDialog();

		int left = ventanaLogin.getAbsoluteLeft() + ((ventanaLogin.getOffsetWidth() - 200) / 2);
		int top = ventanaLogin.getAbsoluteTop() + 80;

		final DecoratedPopupPanel pop = new UsuarioInvalidoAlert(
				"La combinaci\u00F3n de usuario y clave no es correcta!");
		pop.setPopupPosition(left, top);
		pop.show();
	}

	@Override
	public void muestraErrorAccesoProhibido() {
		LoginDialog ventanaLogin = clientFactory.getLoginDialog();

		int left = ventanaLogin.getAbsoluteLeft() + ((ventanaLogin.getOffsetWidth() - 200) / 2);
		int top = ventanaLogin.getAbsoluteTop() + 80;

		final DecoratedPopupPanel pop = new UsuarioInvalidoAlert(
				"El usuario es correcto pero no tiene acceso a las evaluaciones");
		pop.setPopupPosition(left, top);
		pop.show();
	}
}
