package es.upv.client.ui;

import java.util.ArrayList;
import java.util.List;

import com.allen_sauer.gwt.log.client.Log;
import com.smartgwt.client.data.Criteria;
import com.smartgwt.client.data.DataSource;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.types.Alignment;
import com.smartgwt.client.widgets.form.DynamicForm;
import com.smartgwt.client.widgets.form.fields.FormItem;
import com.smartgwt.client.widgets.form.fields.SelectItem;
import com.smartgwt.client.widgets.form.fields.SubmitItem;
import com.smartgwt.client.widgets.form.fields.events.ChangedEvent;
import com.smartgwt.client.widgets.form.fields.events.ChangedHandler;

import es.upv.client.ui.ds.AsignaturaDatasource;
import es.upv.client.ui.ds.CentroDatasource;
import es.upv.client.ui.ds.CursoDatasource;
import es.upv.shared.model.AsignaturaOverlay;
import es.upv.shared.services.TemaAsignaturaServiceAsync;

public class AsignaturaSelector extends DynamicForm {
	
	private static final int SELECT_DEFAULT_ANCHO = 200;

	private SelectItem asignaturaSelect;
	private SelectItem cursoSelect;
	private SelectItem centroSelect;
	
	private boolean permiteMatricular = false;
	
	private ChangedHandler asignaturaSeleccionada;
	
	protected final TemaAsignaturaServiceAsync temaAsignaturaService = TemaAsignaturaServiceAsync.Util
			.getInstance();

	public AsignaturaSelector() {
		initialize(SELECT_DEFAULT_ANCHO);
	}

	public AsignaturaSelector(boolean permiteMatricular) {
		this.permiteMatricular = permiteMatricular;
		
		initialize(SELECT_DEFAULT_ANCHO);
	}

	public void initialize(int anchoSelects) {
		
		// El formulario
		this.setCanSubmit(false);
		this.setNumCols(3);
		this.setWidth100();
		setFixedColWidths(false);
		setColWidths("20%", "*", "20%");

		List<FormItem> campos = obtenerCamposFormulario(this.permiteMatricular, anchoSelects);
		
		this.setFields(campos.toArray(new FormItem[0]));
	}

	protected List<FormItem> obtenerCamposFormulario (boolean permiteMatricular, int anchoSelects) {
		
		List<FormItem> fields = new ArrayList<FormItem>();
		
		DataSource centroDS = new CentroDatasource();
		centroSelect = new SelectItem("centroSelect", "Centro");
		centroSelect.setWidth("*");
		centroSelect.setDisplayField("nombre");
		centroSelect.setValueField("id");
		centroSelect.setOptionDataSource(centroDS);
		centroSelect.setAutoFetchData(true);
		centroSelect.setAllowEmptyValue(true);
		
		fields.add(centroSelect);

		DataSource cursoDS = new CursoDatasource();
		cursoSelect = new SelectItem("cursoSelect", "Curso");
		cursoSelect.setStartRow(true);
		cursoSelect.setWidth("*");
		cursoSelect.setDisplayField("nombre");
		cursoSelect.setValueField("id");
		cursoSelect.setAutoFetchData(true);
		cursoSelect.setAllowEmptyValue(true);
		cursoSelect.setOptionDataSource(cursoDS);
		
		fields.add(cursoSelect);
		
		ChangedHandler filterData = new ChangedHandler() {
			public void onChanged(ChangedEvent event) {
				Log.debug("filtering :: " + cursoSelect.getValueAsString()
						+ " | " + centroSelect.getValueAsString());
				Criteria crt = new Criteria();
				if (cursoSelect.getValueAsString() != null)
					crt.addCriteria("cursoId", cursoSelect.getValueAsString());

				if (centroSelect.getValueAsString() != null)
					crt.addCriteria("centroId", centroSelect.getValueAsString());

				asignaturaSelect.setPickListCriteria(crt);
				asignaturaSelect.fetchData();
				
				asignaturaSelect.clearValue();
				
			}
		};

		cursoSelect.addChangedHandler(filterData);
		centroSelect.addChangedHandler(filterData);
		
		final DataSource asignaturasDS = new AsignaturaDatasource();
		asignaturaSelect = new SelectItem("asignaturaSelect", "Asignatura");
		asignaturaSelect.setStartRow(true);
		asignaturaSelect.setWidth("*");
		asignaturaSelect.setOptionDataSource(asignaturasDS);
		asignaturaSelect.setDisplayField("nombre");
		asignaturaSelect.setValueField("id");
		asignaturaSelect.setFilterLocally(true);
		asignaturaSelect.setAddUnknownValues(false);
		
		asignaturaSelect.addChangedHandler(new ChangedHandler() {
			@Override
			public void onChanged(ChangedEvent event) {
				if (asignaturaSeleccionada != null)
					asignaturaSeleccionada.onChanged(event);
			}
		});
		
		fields.add(asignaturaSelect);

		if (permiteMatricular) {
			final SubmitItem submitItem = new SubmitItem();
			submitItem.setIcon("[SKIN]/actions/add.png");
			submitItem.setTextAlign(Alignment.CENTER);
			submitItem.setTitle("Matricular asignatura");
			submitItem.setStartRow(false);
			submitItem.setDisabled(true);
			
			fields.add(submitItem);
			
			asignaturaSelect.addChangedHandler(new ChangedHandler() {
				@Override
				public void onChanged(ChangedEvent event) {
					Log.debug("onchanged asignatura");
					submitItem.setDisabled(event.getValue() == null);
				}
			});
		}
		
		return fields;
	}

	public Record getAsignaturaSeleccionadaRecord() {
		return asignaturaSelect.getSelectedRecord();
	}
	
	public AsignaturaOverlay getAsignaturaSeleccionada() {
		
		if (asignaturaSelect.getSelectedRecord() != null)
			return asignaturaSelect.getSelectedRecord().getJsObj().cast();
		else 
			return null;
	}
	
	public Integer getAsignaturaSeleccionadaValue () {
		return new Integer ( asignaturaSelect.getValueAsString());
	}

	public ChangedHandler getAsignaturaSeleccionadaHandler() {
		return asignaturaSeleccionada;
	}

	public void setAsignaturaSeleccionadaHandler(
			ChangedHandler asignaturaSeleccionada) {
		this.asignaturaSeleccionada = asignaturaSeleccionada;
	}

}
