package es.upv.client.ui;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.core.client.JsArray;
import com.google.gwt.http.client.Request;
import com.google.gwt.http.client.RequestBuilder;
import com.google.gwt.http.client.RequestCallback;
import com.google.gwt.http.client.RequestException;
import com.google.gwt.http.client.Response;
import com.google.gwt.json.client.JSONException;
import com.google.gwt.json.client.JSONObject;
import com.smartgwt.client.core.DataClass;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.types.Alignment;
import com.smartgwt.client.types.SelectionStyle;
import com.smartgwt.client.util.BooleanCallback;
import com.smartgwt.client.util.JSOHelper;
import com.smartgwt.client.util.SC;
import com.smartgwt.client.widgets.IButton;
import com.smartgwt.client.widgets.Label;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.form.DynamicForm;
import com.smartgwt.client.widgets.form.events.SubmitValuesEvent;
import com.smartgwt.client.widgets.form.events.SubmitValuesHandler;
import com.smartgwt.client.widgets.form.fields.ComboBoxItem;
import com.smartgwt.client.widgets.form.fields.FloatItem;
import com.smartgwt.client.widgets.form.fields.FormItem;
import com.smartgwt.client.widgets.form.fields.SubmitItem;
import com.smartgwt.client.widgets.form.fields.TextAreaItem;
import com.smartgwt.client.widgets.form.fields.TextItem;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridField;
import com.smartgwt.client.widgets.grid.events.CellClickEvent;
import com.smartgwt.client.widgets.grid.events.CellClickHandler;
import com.smartgwt.client.widgets.layout.HLayout;

import es.upv.client.ui.ds.AsignaturaDatasource;
import es.upv.client.utils.AsignaturaServiceUtils;
import es.upv.client.utils.CentroServiceUtils;
import es.upv.client.utils.JsonRequestAction;
import es.upv.shared.model.AsignaturaOverlay;
import es.upv.shared.model.CentroOverlay;
import es.upv.shared.model.CursoOverlay;

public class AsignaturasComponent extends com.smartgwt.client.widgets.layout.VLayout {

	private ComboBoxItem centrosCombo;
	private ComboBoxItem cursosCombo;
	private ListGrid listadoAsignaturas;
	private AsignaturaDatasource asignaturasDS;
	private DynamicForm asignaturaForm;

	public AsignaturasComponent() {

		setAutoHeight();

		Label lblNewLabel_2 = new Label("<h3>Mantenimiento de asignaturas</h3>");
		lblNewLabel_2.setAutoHeight();
		lblNewLabel_2.setAlign(Alignment.CENTER);
		addMember(lblNewLabel_2);

		listadoAsignaturas = new ListGrid();
		listadoAsignaturas.setHeight(200);
		listadoAsignaturas.setSelectionType(SelectionStyle.SINGLE);
		listadoAsignaturas.setCanPickFields(false);
		listadoAsignaturas.setCanGroupBy(false);

		ListGridField fieldCodigo = new ListGridField("codigo", "Código");
		fieldCodigo.setWidth("7%");
		ListGridField fieldNombre = new ListGridField("nombre", "Nombre");
		fieldNombre.setWidth("*");
		
		ListGridField fieldCurso = new ListGridField("cursoNombre", "Curso");
		fieldCurso.setWidth("15%");
		ListGridField fieldCentro = new ListGridField("centroNombre", "Centro");
		fieldCentro.setWidth("30%");

		listadoAsignaturas.setFields(new ListGridField[] { fieldCodigo, fieldNombre, fieldCurso, fieldCentro });
		this.addMember(listadoAsignaturas);

		// botones de acciones
		HLayout buttonLayout = new HLayout();
		buttonLayout.setMargin(5);
		buttonLayout.setMembersMargin(10);
		buttonLayout.setDefaultLayoutAlign(Alignment.RIGHT);

		// boton de eliminar asignatura
		IButton delButton = new IButton("Eliminar asignatura");
		delButton.setWidth(200);
		delButton.setIcon("[SKIN]/actions/remove.png");

		delButton.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {

				if (listadoAsignaturas.getSelectedRecord() != null) {
					final Record r = listadoAsignaturas.getSelectedRecord();
					final String idBorrar = r.getAttributeAsString("id");

					SC.confirm("¿Confirma la eliminación de la asignatura?", new BooleanCallback() {

						@Override
						public void execute(Boolean value) {

							AsignaturaServiceUtils.borrarAsignatura(idBorrar, new JsonRequestAction() {

								@Override
								public void successAction() {
									listadoAsignaturas.removeData(r);
									asignaturaForm.reset();
									asignaturaForm.clearValues();
								}
							});

						}
					});
				}
			}
		});

		buttonLayout.addMember(delButton);

		// accion de nueva asignatura
		IButton newButton = new IButton("Nueva asignatura");
		newButton.setWidth(200);
		newButton.setIcon("[SKIN]/actions/add.png");

		newButton.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {
				listadoAsignaturas.deselectAllRecords();

				asignaturaForm.setGroupTitle("Informaci\u00F3n de la nueva asignatura");
				asignaturaForm.reset();
				asignaturaForm.clearValues();
			}
		});

		buttonLayout.setAlign(Alignment.RIGHT);
		buttonLayout.addMember(newButton);

		addMember(buttonLayout);

		asignaturaForm = new DynamicForm();
		asignaturaForm.setPadding(10);
		asignaturaForm.setIsGroup(true);
		asignaturaForm.setGroupTitle("Información de la asignatura");
		asignaturaForm.setNumCols(4);
		asignaturaForm.setFixedColWidths(false);
		asignaturaForm.setAutoHeight();

		centrosCombo = new ComboBoxItem("centroId", "Centro");
		centrosCombo.setWidth("*");
		cursosCombo = new ComboBoxItem("cursoId", "Curso");
		cursosCombo.setWidth(200);

		listadoAsignaturas.addCellClickHandler(new CellClickHandler() {

			@Override
			public void onCellClick(CellClickEvent event) {
				// dynamicForm.editRecord(listadoAsignaturas.getSelectedRecord());
				asignaturaForm.setGroupTitle("Información de la asignatura");
				asignaturaForm.editSelectedData(listadoAsignaturas);
			}
		});

		// obtener los centros
		actualizarCentros();

		// obtener los cursos
		actualizarCursos();

		SubmitItem submitItem = new SubmitItem();
		submitItem.setColSpan(4);
		submitItem.setAlign(Alignment.RIGHT);
		submitItem.setWidth(200);
		submitItem.setTitle("Guardar");
		submitItem.setIcon("[SKIN]/actions/save.png");

		TextItem itemNombre = new TextItem("nombre", "Nombre");
		itemNombre.setWidth("*");
		itemNombre.setHint("Nombre de la nueva asignatura");
		itemNombre.setShowHintInField(true);

		FormItem itemDescripcion = new TextAreaItem("descripcion", "Descripción");
		itemDescripcion.setWidth("*");
		itemDescripcion.setHeight(60);
		asignaturaForm.setFields(new FormItem[] { itemNombre, new TextItem("codigo", "Código"),
				new TextItem("siglas", "Siglas"), itemDescripcion, centrosCombo, cursosCombo,
				new FloatItem("credTeoria", "Cred. Teoría"), new FloatItem("credLaboratorio", "Cred. Laboratorio"),
				submitItem });

		asignaturaForm.addSubmitValuesHandler(new SubmitValuesHandler() {

			@Override
			public void onSubmitValues(SubmitValuesEvent event) {

				if (asignaturaForm.validate()) {

					// Window.alert("Actualiza record");
					Log.debug(new JSONObject(event.getValues()).toString());

					final Record r2 = listadoAsignaturas.getSelectedRecord();
					final Record valoresGuardar = asignaturaForm.getValuesAsRecord();

					AsignaturaServiceUtils.guardarAsignatura(valoresGuardar, new JsonRequestAction() {

						@Override
						public void successAction(Record datosGuardados) {
							
							Log.debug("Datos guardados : " + new JSONObject(datosGuardados.getJsObj()).toString());

							// datosGuardados.setAttribute("cursoNombre", cursosCombo.getDisplayValue());
							// datosGuardados.setAttribute("centroNombre", centrosCombo.getDisplayValue());

							if (r2 == null || valoresGuardar.getAttribute("id") == null) {
								// es nuevo
								listadoAsignaturas.addData(datosGuardados);
							} else {
								// modficiado
								JSOHelper.apply(datosGuardados.getJsObj(), r2.getJsObj());

								int idx = listadoAsignaturas.getRecordIndex(r2);
								Log.debug("seleccionado " + idx);
								listadoAsignaturas.refreshRow(idx);
							}
							asignaturaForm.setValue("id", datosGuardados.getAttributeAsInt("id"));
							asignaturaForm.setValue("version", datosGuardados.getAttributeAsInt("version"));
						};
					});
				}
			}
		});

		asignaturasDS = new AsignaturaDatasource();
		asignaturaForm.setDataSource(asignaturasDS);

		addMember(asignaturaForm);
	}

	public void actualizarCursos() {
		CentroServiceUtils.obtenerCursos(null, new JsonRequestAction() {
			@Override
			public void resultsAction(List<? extends JavaScriptObject> jscriptObjects) {

				LinkedHashMap<String, String> valueMap = new LinkedHashMap<String, String>();

				for (JavaScriptObject obj : jscriptObjects) {
					CursoOverlay curso = obj.cast();

					valueMap.put(String.valueOf(curso.getId()).toString(), curso.getNombre());
				}

				cursosCombo.setValueMap(valueMap);
			}
		});

	}

	public void actualizarCentros() {

		// Window.alert ("*** actualizando centros");
		if (Log.isDebugEnabled())
			Log.debug("***actualizando centros***");

		CentroServiceUtils.obtenerCentros(null, new JsonRequestAction() {
			@Override
			public void resultsAction(List<? extends JavaScriptObject> jscriptObjects) {

				LinkedHashMap<String, String> valueMap = new LinkedHashMap<String, String>();

				for (JavaScriptObject obj : jscriptObjects) {
					CentroOverlay centro = obj.cast();

					valueMap.put(String.valueOf(centro.getId()).toString(), centro.getNombre());
				}

				centrosCombo.setValueMap(valueMap);
			}
		});

	}

	public void actualizarAsignaturas() {

		// limpiar la lista
		limpiaListado();

		if (Log.isDebugEnabled())
			Log.debug("*** actualizando asignaturas");

		Date now = new Date();
		String url = GWT.getHostPageBaseURL() + "services/api/asignaturas/list.json?now=" + now.getTime();

		RequestBuilder builder = new RequestBuilder(RequestBuilder.GET, url);
		builder.setHeader("Pragma", "no-cache");

		try {
			builder.sendRequest(null, new RequestCallback() {
				public void onError(Request request, Throwable exception) {

				}

				public void onResponseReceived(Request request, Response response) {
					if (200 == response.getStatusCode()) {
						try {
							// parse the response text into JSON
							JsArray<AsignaturaOverlay> data = AsignaturaOverlay.arrayFromJson(response.getText());

							Log.debug(response.getText());

							List<AsignaturaOverlay> cursos = new ArrayList<AsignaturaOverlay>();
							DataClass[] asignaturas = new DataClass[data.length()];
							for (int i = 0; i < data.length(); i++) {
								cursos.add(data.get(i));

								Record r = new Record(data.get(i));

								Log.debug("Asignatura: " + Arrays.toString(r.getAttributes()));
								Log.debug("Asignatura: " + String.valueOf(r.getAttribute("cursoNombre")));
								Log.debug("Asignatura: " + data.get(i).getCurso());

								// action for one record
								listadoAsignaturas.addData(r);
								asignaturas[i] = new DataClass(data.get(i));
							}

							asignaturasDS.setTestData(asignaturas);

						} catch (JSONException e) {
						}
					} else {

					}
				}
			});
		} catch (RequestException e) {

		}
	}

	public void limpiaListado() {
		this.listadoAsignaturas.setData(new Record[] {});
		asignaturaForm.reset();
		asignaturaForm.clearValues();
	}

	public AsignaturaDatasource getDs() {
		return asignaturasDS;
	}

	public void setDs(AsignaturaDatasource ds) {
		this.asignaturasDS = ds;
	}

}
