package es.upv.client.ui;

import java.util.List;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.widgets.Label;
import com.smartgwt.client.widgets.form.DynamicForm;
import com.smartgwt.client.widgets.form.fields.events.ChangedEvent;
import com.smartgwt.client.widgets.form.fields.events.ChangedHandler;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridRecord;
import com.smartgwt.client.widgets.grid.events.RecordDoubleClickEvent;
import com.smartgwt.client.widgets.grid.events.RecordDoubleClickHandler;
import com.smartgwt.client.widgets.layout.VLayout;

import es.upv.client.ui.ds.EvaluacionAlumnoDatasource;
import es.upv.shared.dto.EvaluacionAlumno;
import es.upv.shared.dto.RespuestaAlumno;
import es.upv.shared.services.EvaluacionServiceAsync;

public class CorreccionesComponent extends VLayout {

	private EvaluacionAlumnoDatasource ds = new EvaluacionAlumnoDatasource();
	private final static EvaluacionServiceAsync evaluacionService = EvaluacionServiceAsync.Util.getInstance();

	public CorreccionesComponent() {

		Label lblevaluacionesRealizadas = new Label("<strong>Evaluaciones realizadas</strong>");
		lblevaluacionesRealizadas.setHeight("25px");
		addMember(lblevaluacionesRealizadas);

		final AsignaturaSelector asignaturaSelector = new AsignaturaSelector();

		DynamicForm dynamicForm = new DynamicForm();

		final ListGrid listadoEvaluaciones = new ListGrid();
		listadoEvaluaciones.setDataSource(ds);

		asignaturaSelector.setAsignaturaSeleccionadaHandler(new ChangedHandler() {

			@Override
			public void onChanged(ChangedEvent event) {
				/*
				 * crit.addCriteria("asignatura_id",
				 * asignaturaSelector.getAsignaturaSeleccionadaValue());
				 * listadoEvaluaciones.fetchData(crit);
				 */

				evaluacionService.obtenerEvaluacionesRealizadasAsignatura(
						new Long(asignaturaSelector.getAsignaturaSeleccionadaValue()),
						new AsyncCallback<List<EvaluacionAlumno>>() {

							@Override
							public void onFailure(Throwable caught) {
								// TODO Auto-generated method stub

							}

							@Override
							public void onSuccess(List<EvaluacionAlumno> result) {
								if (result != null) {

									Log.debug("N evaluaciones: " + result.size());

									// limpiar grid y aadir preguntas al GRID
									Record[] evaluaciones = new Record[result.size()];
									for (int i = 0; i < result.size(); i++) {

										evaluaciones[i] = new ListGridRecord();
										evaluaciones[i].setAttribute("id", result.get(i).getId().intValue());
										evaluaciones[i].setAttribute("estado", result.get(i));
										evaluaciones[i].setAttribute("asignaturaID", result.get(i).getAsignaturaID());
										evaluaciones[i].setAttribute("alumnoDNI", result.get(i).getAlumno().getDni());
										evaluaciones[i].setAttribute("alumnoNombre", result.get(i).getAlumno()
												.getPrimerNombre()
												+ " " + result.get(i).getAlumno().getApellidoPaterno());
										evaluaciones[i].setAttribute("fecha", result.get(i).getFecha());

									}

									listadoEvaluaciones.setData(evaluaciones);

								} else
									Window.alert("sin resultados");
							}

						});
			}
		});

		listadoEvaluaciones.addRecordDoubleClickHandler(new RecordDoubleClickHandler() {

			@Override
			public void onRecordDoubleClick(final RecordDoubleClickEvent event) {
				// abrir el panel de correcciones
				final PanelCorrecciones panel = new PanelCorrecciones();

				panel.setWidth("80%");
				panel.setHeight("80%");

				// obtener las respuestas
				evaluacionService.obtenerPreguntasEvaluacionAlumno(new Long(event.getRecord().getAttributeAsInt("id")),
						new AsyncCallback<List<RespuestaAlumno>>() {
							public void onFailure(Throwable caught) {
							};

							public void onSuccess(java.util.List<RespuestaAlumno> result) {
								
								Log.debug("n de respuestas : " + result.size());
								panel.defineRespuestas(result);
								panel.defineEvaluacionAlumnoID(event.getRecord().getAttributeAsInt("id"));

								panel.show();
							};
						});
			}
		});

		addMember(asignaturaSelector);
		addMember(dynamicForm);
		addMember(listadoEvaluaciones);
	}

}
