package es.upv.client.ui;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.smartgwt.client.core.DataClass;
import com.smartgwt.client.data.Criteria;
import com.smartgwt.client.data.DSCallback;
import com.smartgwt.client.data.DSRequest;
import com.smartgwt.client.data.DSResponse;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.types.Alignment;
import com.smartgwt.client.types.Overflow;
import com.smartgwt.client.util.JSOHelper;
import com.smartgwt.client.widgets.EdgedCanvas;
import com.smartgwt.client.widgets.IButton;
import com.smartgwt.client.widgets.Label;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.form.DynamicForm;
import com.smartgwt.client.widgets.form.fields.CheckboxItem;
import com.smartgwt.client.widgets.form.fields.DateItem;
import com.smartgwt.client.widgets.form.fields.DateTimeItem;
import com.smartgwt.client.widgets.form.fields.FormItem;
import com.smartgwt.client.widgets.form.fields.HiddenItem;
import com.smartgwt.client.widgets.form.fields.IntegerItem;
import com.smartgwt.client.widgets.form.fields.TextItem;
import com.smartgwt.client.widgets.form.fields.events.ChangedEvent;
import com.smartgwt.client.widgets.form.fields.events.ChangedHandler;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.layout.HStack;
import com.smartgwt.client.widgets.layout.LayoutSpacer;

import es.upv.client.ui.ds.EvaluacionDatasource;
import es.upv.client.ui.ds.EvaluacionPreguntasDatasource;
import es.upv.client.ui.ds.PreguntasDatasource;
import es.upv.shared.dto.Evaluacion;
import es.upv.shared.dto.EvaluacionPregunta;
import es.upv.shared.dto.Pregunta;
import es.upv.shared.services.EvaluacionServiceAsync;
import es.upv.shared.services.TemaAsignaturaServiceAsync;

public class EvaluacionesComponent extends com.smartgwt.client.widgets.layout.VLayout {
	
	/**
	 * Create a remote service proxy to talk to the server-side Greeting
	 * service.
	 */ 
	private final TemaAsignaturaServiceAsync temaAsignaturaService = TemaAsignaturaServiceAsync.Util.getInstance();

	private final EvaluacionServiceAsync evaluacionService = EvaluacionServiceAsync.Util.getInstance();

	private static EvaluacionPreguntasDatasource evaluacionesDatasource = new EvaluacionPreguntasDatasource();

	public EvaluacionesComponent() {

		this.setMembersMargin(5);
		setPadding(10);

		IButton newButton = new IButton("Nueva evaluacion");
		newButton.setLayoutAlign(Alignment.CENTER);
		newButton.setSize("200px", "22px");
		newButton.setIcon("[SKIN]/actions/plus.png");
		addMember(newButton);
		newButton.moveTo(200, 0);

		LayoutSpacer layoutSpacer = new LayoutSpacer();
		layoutSpacer.setHeight("21px");
		addMember(layoutSpacer);

		final TemaAsignaturaSelector temaSelector = new TemaAsignaturaSelector(false);
		temaSelector.setTemaSeleccionadoHandler(new ChangedHandler() {
			public void onChanged(ChangedEvent event) {

				// mostrar preguntas de un tema en el listGrid
				temaAsignaturaService.obtenerPreguntasTema(new Long(event.getValue().toString()),
						new AsyncCallback<List<Pregunta>>() {
							public void onSuccess(List<Pregunta> result) {
								if (result != null)
									Window.alert("" + result.size());
								else
									Window.alert("sin resultados");
							};

							@Override
							public void onFailure(Throwable caught) {
								Log.debug("Error obteniendo temas");
							};
						});
			}
		});

		addMember(temaSelector);

		final IButton addPreguntasButton = new IButton();
		addPreguntasButton.setTitle("Añadir preguntas del tema a la evaluación");
		addPreguntasButton.setAutoFit(true);
		addPreguntasButton.setIcon("[SKIN]/actions/add.png");
		addPreguntasButton.setLayoutAlign(Alignment.CENTER); // .setStyleName("paddingLeft");
		addMember(addPreguntasButton);

		Label lblPreguntas = new Label("<strong>Preguntas</strong>");
		lblPreguntas.setHeight("24px");
		addMember(lblPreguntas);

		final ListGrid listadoPreguntas = new ListGrid();
		listadoPreguntas.setDataSource(evaluacionesDatasource);
		listadoPreguntas.setCanEdit(true);
		listadoPreguntas.setCanRemoveRecords(true);
		listadoPreguntas.setAnimateRemoveRecord(false);
		listadoPreguntas.setEditByCell(true);
		listadoPreguntas.setEditOnFocus(true);
		listadoPreguntas.setAutoFetchData(true);
		listadoPreguntas.setHeight(1);
		listadoPreguntas.setTop(50);
		listadoPreguntas.setShowAllRecords(true);
		listadoPreguntas.setBodyOverflow(Overflow.VISIBLE);
		listadoPreguntas.setOverflow(Overflow.VISIBLE);
		listadoPreguntas.setLeaveScrollbarGap(false);

		final PreguntasDatasource preguntasDS = new PreguntasDatasource();

		temaSelector.setTemaSeleccionadoHandler(new ChangedHandler() {
			public void onChanged(ChangedEvent event) {

			}
		});

		addMember(listadoPreguntas);

		final DynamicForm evaluacionForm = new DynamicForm();
		evaluacionForm.setDataSource(new EvaluacionDatasource());

		evaluacionForm.setWidth("386px");
		TextItem textItem = new TextItem("titulo", "Título");
		textItem.setLength(300);
		textItem.setWidth(500);
		DateItem fItem = new DateItem("fechaApertura", "Fecha de apertura");
		fItem.setWrapTitle(false);
		HiddenItem asigID = new HiddenItem("asignatura");
		HiddenItem temaID = new HiddenItem("tema");
		evaluacionForm.setFields(new FormItem[] { textItem, fItem, new DateTimeItem("fechaCierre", "Fecha de cierre"),
				new IntegerItem("n_preguntas", "Número de preguntas"), new CheckboxItem("publicado", "Publicado"),
				asigID, temaID });
		addMember(evaluacionForm);

		EdgedCanvas edged = new EdgedCanvas();

		HStack hStack = new HStack();
		hStack.setMembersMargin(20);

		final IButton guardarEvaluacionButton = new IButton();
		guardarEvaluacionButton.setIcon("[SKIN]/actions/ok.png");
		guardarEvaluacionButton.setWidth(200);
		guardarEvaluacionButton.setTitle("Guardar evaluación");

		final Label lblEvaluacinNoGuardada = new Label("Evaluación no guardada");
		// lblEvaluacinNoGuardada.setSize("447px", "23px");
		lblEvaluacinNoGuardada.setHeight(20);
		lblEvaluacinNoGuardada.setWrap(false);
		hStack.addMember(guardarEvaluacionButton);
		hStack.addMember(lblEvaluacinNoGuardada);

		addPreguntasButton.addClickHandler(new com.smartgwt.client.widgets.events.ClickHandler() {

			@Override
			public void onClick(com.smartgwt.client.widgets.events.ClickEvent event) {
				// mostrar preguntas de un tema en el listGrid
				// listadoPreguntas.setData(new ListGridRecord[] {} );
				Criteria criteria = new Criteria();
				criteria.addCriteria("tema", temaSelector.getTemaSeleccionado());
				preguntasDS.fetchData(criteria, new DSCallback() {

					@Override
					public void execute(DSResponse response, Object rawData, DSRequest request) {
						Log.debug("obteniendo preguntas ");
						for (String str : JSOHelper.getProperties(response.getData()[0].getJsObj()))
							Log.debug(str);

						for (Record r : response.getData()) {
							listadoPreguntas.addData(r);
						}

						evaluacionForm.setValue("asignatura", temaSelector.getAsignaturaSeleccionadaValue());
						evaluacionForm.setValue("tema", temaSelector.getTemaSeleccionado());
					}
				});
			}
		});

		guardarEvaluacionButton.addClickHandler(new com.smartgwt.client.widgets.events.ClickHandler() {

			@Override
			public void onClick(com.smartgwt.client.widgets.events.ClickEvent event) {

				if (evaluacionForm.validate()) {

					Log.debug("Guardando " + listadoPreguntas.getTotalRows() + " preguntas");

					Evaluacion nuevaEvaluacion = new Evaluacion();
					nuevaEvaluacion.setTitulo(evaluacionForm.getValueAsString("titulo"));

					nuevaEvaluacion.setPublicado((Boolean) evaluacionForm.getValue("publicado") != null);
					nuevaEvaluacion.setFechaApertura((Date) evaluacionForm.getValue("fechaApertura"));
					nuevaEvaluacion.setFechaCierre((Date) evaluacionForm.getValue("fechaCierre"));
					nuevaEvaluacion.setNumeroPreguntas(new Integer(evaluacionForm.getValueAsString("n_preguntas")));
					nuevaEvaluacion.setTemaID(new Long((Integer) evaluacionForm.getValue("tema")));
					nuevaEvaluacion.setAsignaturaID(new Long((Integer) evaluacionForm.getValue("asignatura")));

					evaluacionService.guardarEvaluacion(nuevaEvaluacion, new AsyncCallback<Evaluacion>() {
						@Override
						public void onFailure(Throwable caught) {
							lblEvaluacinNoGuardada.setContents("ERROR AL GUARDAR LA EVALUACION");
						}

						@Override
						public void onSuccess(Evaluacion result) {

							// FIXME : GUARDAR LAS PREGUNTAS!!!
							List<EvaluacionPregunta> preguntas = new ArrayList<EvaluacionPregunta>();
							for (Record r : listadoPreguntas.getRecords()) {
								EvaluacionPregunta preg = new EvaluacionPregunta();
								preg.setPreguntaID(new Long(r.getAttributeAsString("preguntaId")));
								preg.setTemaID(new Long(r.getAttributeAsInt("tema")));
								preg.setEvaluacionID(result.getId());
								preg.setDificultad(0.0f);

								preguntas.add(preg);
							}

							evaluacionService.guardarPreguntasEvaluacion(result.getId(), preguntas,
									new AsyncCallback<Void>() {

										@Override
										public void onFailure(Throwable caught) {
											lblEvaluacinNoGuardada
													.setContents("Error al guardar las preguntas --> eliminar evaluación");
										}

										@Override
										public void onSuccess(Void result) {
										}
									});

							listadoPreguntas.setDisabled(true);
							evaluacionForm.setDisabled(true);
							addPreguntasButton.setDisabled(true);
							guardarEvaluacionButton.setDisabled(true);
							lblEvaluacinNoGuardada.setContents("Evaluaci\u00F3n guardada!!");

						}
					});
				}
			}
		});

		newButton.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {

				listadoPreguntas.setData(new Record[] {});
				evaluacionesDatasource.setTestData(new DataClass[] {});
				listadoPreguntas.fetchData();
				// listadoPreguntas.fetchData(new Criteria ("preguntaId",
				// "99"));
				evaluacionForm.resetValues();
				listadoPreguntas.setDisabled(false);
				evaluacionForm.setDisabled(false);
				addPreguntasButton.setDisabled(false);
				guardarEvaluacionButton.setDisabled(false);
				lblEvaluacinNoGuardada.setContents("Evaluación no guardada");
			}

		});

		hStack.setPadding(10);
		edged.setHeight(55);
		edged.setLayoutAlign(Alignment.CENTER);
		edged.addChild(hStack);
		addMember(edged);

		// initWidget(this);
	}
}
