package es.upv.client.ui;

import com.allen_sauer.gwt.log.client.Log;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.widgets.Label;
import com.smartgwt.client.widgets.layout.HLayout;
import com.smartgwt.client.widgets.tab.Tab;
import com.smartgwt.client.widgets.tab.TabSet;
import com.smartgwt.client.widgets.tab.events.TabSelectedEvent;
import com.smartgwt.client.widgets.tab.events.TabSelectedHandler;

import es.upv.client.ui.events.CentrosChangedEvent;
import es.upv.client.ui.events.CentrosChangedEventHandler;
import es.upv.client.utils.CentroServiceUtils;
import es.upv.client.utils.JsonRequestAction;

/**
 * Entry point classes define <code>onModuleLoad()</code>.
 */
public class InfoAcademica extends HLayout implements InfoAcademicaView {
	@SuppressWarnings("unused")
	private Presenter listener;
	
	private String defaultTab = "centros";

	/**
	 * Create a remote service proxy to talk to the server-side Greeting
	 * service.
	 */
	private Label lblNewLabel_4;
	private CentrosComponent centrosComponent;
	private AsignaturasComponent asignaturasComponent;
	private MatriculasComponent matriculasComponent;
	private AutoresComponent autoresComponent;
	
	private TabSet tabSet;

	/**
	 * This is the entry point method.
	 */
	public InfoAcademica() {
		Log.debug("InfoAcademica!!");
		
		setWidth("98%");
		// hStack.setHeight100();

		tabSet = new TabSet();
		tabSet.setWidth100();
		tabSet.setHeight(700);
		
		tabSet.addTabSelectedHandler(new TabSelectedHandler() {
			
			@Override
			public void onTabSelected(TabSelectedEvent event) {
				// listener.goTo(new PanelAcademicaPlace(event.getID()));
			}
		});

		// *****************************************************************************************
		// Pestaña de centros **********************************************************************
		// *****************************************************************************************

		Tab tab_1 = new Tab("Centros");
		tab_1.setID("centros");
		tab_1.addTabSelectedHandler(new TabSelectedHandler() {
			public void onTabSelected(TabSelectedEvent event) {
				// actualizar centros
				centrosComponent.limpiaCentros();

				CentroServiceUtils.obtenerCentros(lblNewLabel_4, new JsonRequestAction() {
					@Override
					public void actionRecord(Record r) {
						centrosComponent.addCentro(r);
					}

				});
			}
		});

		centrosComponent = new CentrosComponent();
		tab_1.setPane(centrosComponent);
		tabSet.addTab(tab_1);

		// *****************************************************************************************
		// Pestaña de asignaturas ******************************************************************
		// *****************************************************************************************

		Tab tab_2 = new Tab("Asignaturas");
		tab_2.setID("asignaturas");
		tab_2.addTabSelectedHandler(new TabSelectedHandler() {

			public void onTabSelected(TabSelectedEvent event) {
				asignaturasComponent.actualizarAsignaturas();
			}
		});

		asignaturasComponent = new AsignaturasComponent();

		Root.EVENT_BUS.addHandler(CentrosChangedEvent.TYPE, new CentrosChangedEventHandler() {

			@Override
			public void onCentrosChanged(CentrosChangedEvent event) {
				asignaturasComponent.actualizarCentros();
			}
		});

		tab_2.setPane(asignaturasComponent);
		tabSet.addTab(tab_2);

		// *****************************************************************************************
		// Pestaña de matriculas *******************************************************************
		// *****************************************************************************************

		Tab tabMatriculas = new Tab("Alumnos y matr\u00EDculas");
		tabMatriculas.setID("matriculas");
		matriculasComponent = new MatriculasComponent(asignaturasComponent.getDs());

		tabMatriculas.addTabSelectedHandler(new TabSelectedHandler() {

			public void onTabSelected(TabSelectedEvent event) {
				matriculasComponent.actualizarAlumnos();
			}
		});
		tabMatriculas.setPane(matriculasComponent);
		tabSet.addTab(tabMatriculas);
		
		Tab tabAutores = new Tab("Autores");
		tabAutores.setID("autores");
		autoresComponent = new AutoresComponent();

		tabAutores.addTabSelectedHandler(new TabSelectedHandler() {

			public void onTabSelected(TabSelectedEvent event) {
				autoresComponent.actualizarAutores();
			}
		});
		tabAutores.setPane(autoresComponent);
		tabSet.addTab(tabAutores);

		// tabSet.selectTab(defaultTab);
		
		Log.debug("defaulttab : " + defaultTab);
		addMember(tabSet);
		// hStack.draw();
	}

	@Override
	public void setName(String tabID) {
		Log.debug("Setting infoAcademica::" + tabID);
		
		defaultTab = tabID;
	}

	@Override
	public void setPresenter(Presenter listener) {
		this.listener = listener;
	}
}
