package es.upv.client.ui;

import java.util.List;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.StatusCodeException;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.types.Alignment;
import com.smartgwt.client.types.AnimationEffect;
import com.smartgwt.client.util.SC;
import com.smartgwt.client.widgets.IButton;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridField;
import com.smartgwt.client.widgets.grid.ListGridRecord;
import com.smartgwt.client.widgets.layout.HLayout;
import com.smartgwt.client.widgets.layout.VLayout;

import es.upv.client.security.web.EvalticsForbiddenException;
import es.upv.client.ui.events.UsuarioValidadoEvent;
import es.upv.client.ui.events.UsuarioValidadoEventHandler;
import es.upv.shared.dto.Evaluacion;
import es.upv.shared.dto.EvaluacionAlumno;
import es.upv.shared.dto.RespuestaAlumno;
import es.upv.shared.services.EvaluacionServiceAsync;

public class PanelAlumnos extends VLayout implements PanelAlumnoView {
	private Presenter listener;

	private EvaluacionServiceAsync evaluacionService = EvaluacionServiceAsync.Util.getInstance();

	private Record alumno;
	private ListGrid lg;

	private PanelRespuesta panelRespuestas;

	PanelAlumnos thisPanel = null;

	public PanelAlumnos() {
		thisPanel = this;

		Log.debug("PanelAlumnos!!");

		setWidth("80%");
		setAutoHeight();
		setMembersMargin(10);

		final HLayout layout = new HLayout();

		layout.setAlign(Alignment.CENTER);
		layout.setLayoutMargin(20);
		layout.setShowEdges(true);

		lg = new ListGrid();
		lg.setEmptyMessage("");
		layout.setHeight(200);
		ListGridField fEvID = new ListGridField("id");
		fEvID.setHidden(true);
		ListGridField fEvID2 = new ListGridField("idEvAlumno");
		fEvID.setHidden(true);
		ListGridField fTitulo = new ListGridField("titulo", "Evaluación");
		ListGridField fEstado = new ListGridField("estado", "Estado");
		ListGridField fEstado2 = new ListGridField("estadoDisplay", "Estado");
		ListGridField fFecha = new ListGridField("fecha", "Fecha comienzo");
		ListGridField fFechaMax = new ListGridField("fechaCierre", "Fecha máxima");

		lg.setFields(fEvID, fEvID2, fTitulo, fEstado, fEstado2, fFecha, fFechaMax);

		// layout.setVisible(false);

		Root.EVENT_BUS.addHandler(UsuarioValidadoEvent.TYPE, new UsuarioValidadoEventHandler() {

			@Override
			public void onUsuarioValidado(UsuarioValidadoEvent event) {
				Log.debug("Usuario correctamente validado");

				listener.ocultarDialogoLogin();
				
				alumno = event.getAlumno();

				lg.setEmptyMessage("Sin evaluaciones pendientes");

				Element nombreUsuario = DOM.getElementById("nombreUsuario");
				nombreUsuario.setInnerHTML("<em>( " + alumno.getAttribute("primerNombre") + " "
						+ alumno.getAttribute("apellidoPaterno") + " )</em>");

				Element disconLink = DOM.getElementById("disconectLink");
				disconLink.setInnerHTML("Desconectar");

				rellenarEvaluacionesAlumno(alumno.getAttributeAsInt("id"));
			}

			@Override
			public void onUsuarioIncorrecto(UsuarioValidadoEvent event) {
				Log.debug("usuario incorrecto");

				listener.muestraErrorCredenciales ();
			}
		});

		layout.addMember(lg);

		IButton bi = new IButton("Comenzar evaluación");
		bi.setLayoutAlign(Alignment.CENTER);
		bi.setAutoFit(true);
		bi.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {

				Record r = lg.getSelectedRecord();

				// obtener el estado
				EstadoEvaluacion estado = (EstadoEvaluacion) r.getAttributeAsObject("estado");
				if (EstadoEvaluacion.ESTADO_FINALIZADO.equals(estado)) {
					SC.warn("Evaluación finalizada, no se puede continuar");
				} else

				if (r.getAttribute("idEvAlumno") != null) {
					// existe la evaluacion de alumno... obtener preguntas
					// y continuar por la primera no contestada
					Log.debug("Evaluacion ya comenzada");

					comienzaEvaluacion(r.getAttribute("idEvAlumno"));

				} else {
					// comenza una nueva evaluacion
					Log.debug("Nueva evaluacion");
					evaluacionService.comenzarEvaluacion(new Long(obtenerIDAlumno()), new Long(r.getAttribute("id")),
							new AsyncCallback<EvaluacionAlumno>() {
								public void onSuccess(EvaluacionAlumno result) {

									comienzaEvaluacion(result.getId().toString());

									rellenarEvaluacionesAlumno(new Integer(result.getAlumnoID().toString()));
								};

								@Override
								public void onFailure(Throwable caught) {
									SC.warn("Error obteniendo la evalucion");
								}
							});

				}
			}
		});

		IButton bFinalizar = new IButton("Finalizar evaluación");
		bFinalizar.setLayoutAlign(Alignment.CENTER);
		bFinalizar.setAutoFit(true);
		bFinalizar.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {

				Record r = lg.getSelectedRecord();

				if (r.getAttribute("idEvAlumno") == null) {
					// existe la evaluacion de alumno... obtener preguntas
					// y continuar por la primera no contestada
					Log.debug("Evaluacion no comenzada ! ");

				} else {
					// finaliza una evaluacion
					Log.debug("finaliza evaluacion");
					evaluacionService.finalizarEvaluacion(new Long(r.getAttributeAsString("idEvAlumno")),
							new AsyncCallback<Void>() {

								public void onFailure(Throwable caught) {
									SC.say("Error al finalizar");
								}

								@Override
								public void onSuccess(Void result) {
									SC.say("Evaluación finalizada");
									rellenarEvaluacionesAlumno(obtenerIDAlumno());
								};
							});
				}
			}
		});

		addMember(layout);
		addMember(bi);
		addMember(bFinalizar);

		panelRespuestas = new PanelRespuesta();
		panelRespuestas.setWidth("80%");
		panelRespuestas.setHeight("80%");
	}

	protected void comienzaEvaluacion(String evaluacionID) {

		// obtener las preguntas de la evaluacion y mostrar el panel de
		// respuestas
		evaluacionService.obtenerPreguntasEvaluacionAlumno(new Long(evaluacionID),
				new AsyncCallback<List<RespuestaAlumno>>() {
					@Override
					public void onFailure(Throwable caught) {
						SC.say("No se pueden obtener las pregunta");
					}

					@Override
					public void onSuccess(List<RespuestaAlumno> result) {

						Log.debug("obtenidas " + result.size() + " preguntas");

						panelRespuestas.definePreguntas(result);
						panelRespuestas.comienzaEvaluacion();

						panelRespuestas.animateShow(AnimationEffect.FLY);
					}
				});

	}

	protected void rellenarEvaluacionesAlumno(final Integer alumnoId) {

		lg.setData(new Record[] {});

		// obtener todas las evaluaciones pendientes del alumno
		evaluacionService.obtenerEvaluacionesPendientesAlumno(new Long(alumnoId),
				new AsyncCallback<List<Evaluacion>>() {

					@Override
					public void onSuccess(List<Evaluacion> result) {
						// para cada una de las evaluaciones, ver en que estado
						// está
						for (final Evaluacion evaluacion : result) {
							evaluacionService.obtenerEvaluacionAlumno(new Long(alumnoId), evaluacion.getId(),
									new AsyncCallback<EvaluacionAlumno>() {
										public void onFailure(Throwable caught) {
											SC.say("error al obtener el estado de la evaluacion " + evaluacion.getId());
										};

										@Override
										public void onSuccess(EvaluacionAlumno result) {
											// si existe, el estado es comenzado
											ListGridRecord r = new ListGridRecord();

											EstadoEvaluacion estado = EstadoEvaluacion.ESTADO_SIN_COMENZAR;

											if (result != null && result.getId() != null) {

												r.setAttribute("idEvAlumno", result.getId());
												r.setAttribute("fecha", result.getFecha());

												if (result.getFinalizado())
													estado = EstadoEvaluacion.ESTADO_FINALIZADO;
												else
													estado = EstadoEvaluacion.ESTADO_COMENZADO;
											}

											r.setAttribute("id", evaluacion.getId());
											r.setAttribute("titulo", evaluacion.getTitulo());
											r.setAttribute("estado", estado);
											r.setAttribute("estadoDisplay", estado.getDisplay());
											r.setAttribute("fechaCierre", evaluacion.getFechaCierre());

											lg.addData(r);
										}
									});

						}

					}

					@Override
					public void onFailure(Throwable caught) {
						Log.debug(caught.getLocalizedMessage());
						Log.debug(caught.getClass().toString());

						if (caught instanceof EvalticsForbiddenException) {

							Log.debug("Acceso prohibido para alumnos");
							listener.mostrarDialogoLogin();
							listener.muestraErrorAccesoProhibido();

						} else if (caught instanceof StatusCodeException) {
							StatusCodeException ex = (StatusCodeException) caught;
							Log.debug(String.valueOf(ex.getStatusCode()));

							if (ex.getStatusCode() == 500) {

								listener.mostrarDialogoLogin();

							} else
								SC.warn("Error al obtener las evaluaciones");

						} else
							SC.warn("Error al obtener las evaluaciones");
					}
				});

	}

	protected int obtenerIDAlumno() {
		if (alumno != null && alumno.getAttribute("id") != null)
			return new Integer(alumno.getAttribute("id"));

		return -1;
	}

	@Override
	public void setName(String helloName) {
		Log.debug("Setting panelAlumnos::name");
	}

	@Override
	public void setPresenter(Presenter listener) {
		this.listener = listener;
	}

	@Override
	public void refrescarEvaluaciones() {
		// test
		// rellenarEvaluacionesAlumno(obtenerIDAlumno());
		if (obtenerIDAlumno() > 0) {
			Log.debug("Alumno actual " + obtenerIDAlumno());

			rellenarEvaluacionesAlumno(obtenerIDAlumno());
		} else {
			listener.mostrarDialogoLogin();
		}
	}
	
	@Override
	public void limpiaAlumno () {
		alumno = null;
	}

}
