package es.upv.client.ui;

import java.util.LinkedHashMap;
import java.util.List;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.types.Alignment;
import com.smartgwt.client.types.AnimationEffect;
import com.smartgwt.client.util.SC;
import com.smartgwt.client.widgets.Canvas;
import com.smartgwt.client.widgets.IButton;
import com.smartgwt.client.widgets.Label;
import com.smartgwt.client.widgets.Window;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.form.DynamicForm;
import com.smartgwt.client.widgets.form.fields.FormItem;
import com.smartgwt.client.widgets.form.fields.RadioGroupItem;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridField;
import com.smartgwt.client.widgets.grid.ListGridRecord;
import com.smartgwt.client.widgets.grid.events.RecordClickEvent;
import com.smartgwt.client.widgets.grid.events.RecordClickHandler;
import com.smartgwt.client.widgets.layout.HLayout;
import com.smartgwt.client.widgets.layout.VLayout;

import es.upv.client.ui.correccion.PanelCorreccionOM;
import es.upv.client.ui.correccion.PanelCorreccionRL;
import es.upv.client.ui.correccion.PanelCorreccionRespuesta;
import es.upv.client.ui.correccion.PanelCorreccionVF;
import es.upv.shared.dto.RespuestaAlumno;
import es.upv.shared.dto.RespuestaPregunta;
import es.upv.shared.services.EvaluacionServiceAsync;
import es.upv.shared.services.PreguntasServiceAsync;

public class PanelCorrecciones extends Window {

	private PreguntasServiceAsync preguntasService = PreguntasServiceAsync.Util.getInstance();
	private EvaluacionServiceAsync evaluacionService = EvaluacionServiceAsync.Util.getInstance();

	private int preguntaActual;
	private List<RespuestaAlumno> respuestasEvaluacion;

	private PanelCorreccionRespuesta panelMuestraRespuesta;

	private ListGrid listadoRespuestas;
	private Canvas canvas;
	private Integer evaluacion;
	private Label lblEnunciado;
	private RadioGroupItem radioGroupItem; 

	public PanelCorrecciones() {
		setAutoCenter(true);
		setShowMinimizeButton(false);
		setShowHeader(true);
		setShowEdges(false);
		setShowResizeBar(true);
		setShowTitle(true);
		setShowHeaderIcon(false);
		setShowModalMask(true);
		setAnimateShowEffect(AnimationEffect.FADE);
		setModalMaskOpacity(30);
		setIsModal(true);
		setShowShadow(true);

		HLayout hLayout = new HLayout();
		hLayout.setWidth("786px");
		hLayout.setMembersMargin(30);

		listadoRespuestas = new ListGrid();
		listadoRespuestas.setSize("65%", "*");
		listadoRespuestas.setCanReorderFields(false);
		listadoRespuestas.setCanReorderRecords(false);
		
		hLayout.addMember(listadoRespuestas);
		listadoRespuestas.setFields(new ListGridField("id"), new ListGridField("titulo"), new ListGridField("tipo"),
				new ListGridField("nota"));

		VLayout layout = new VLayout();
		layout.setSize("*", "470px");
		layout.setMembersMargin(30);

		lblEnunciado = new Label("Enunciado de la pregunta : <br>");
		lblEnunciado.setWidth100();

		layout.addMember(lblEnunciado);

		canvas = new Canvas();
		canvas.setSize("100%", "*");
		layout.addMember(canvas);

		Label lblLaRespuestaEs = new Label("Marcar la respuesta como correcta o incorrecta ?");
		lblLaRespuestaEs.setHeight("27px");
		layout.addMember(lblLaRespuestaEs);

		DynamicForm dynamicForm = new DynamicForm();
		radioGroupItem = new RadioGroupItem("respuesta", "");
		radioGroupItem.setShowTitle(false);
		radioGroupItem.setTextAlign(Alignment.CENTER);
		radioGroupItem.setAlign(Alignment.CENTER);

		LinkedHashMap<String, String> values = new LinkedHashMap<String, String>();
		values.put("1", "Correcta");
		values.put("0", "Incorrecta");
		radioGroupItem.setValueMap(values);
		dynamicForm.setFields(new FormItem[] { radioGroupItem });
		layout.addMember(dynamicForm);

		HLayout hLayout_1 = new HLayout();
		hLayout_1.setHeight("5%");
		hLayout_1.setMembersMargin(50);

		IButton btnAnterior = new IButton("Anterior");
		btnAnterior.setSize("*", "50px");
		btnAnterior.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				anteriorPregunta();
			}
		});

		hLayout_1.addMember(btnAnterior);

		IButton btnGrabarNota = new IButton("Grabar<br>Nota");
		btnGrabarNota.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {

				if (radioGroupItem.getValue() != null) {

					RespuestaAlumno respuestaActual = respuestasEvaluacion.get(preguntaActual);

					final Integer nota = Integer.valueOf(radioGroupItem.getValueAsString());
					respuestaActual.setNota(nota.floatValue());

					evaluacionService.guardaRespuesta(new Long(evaluacion), respuestaActual, new AsyncCallback<RespuestaAlumno>() {
						@Override
						public void onFailure(Throwable caught) {
							SC.warn("Error guardando la nota");

						}

						@Override
						public void onSuccess(RespuestaAlumno result) {
							SC.warn("Nota guardada!");
							
							// actualizar la nota en el registro del listado
							Record r = listadoRespuestas.getRecord(preguntaActual);
							
							r.setAttribute("nota", nota);
							r.setAttribute("version", result.getVersion());
							
							listadoRespuestas.refreshRow(preguntaActual);
							
							// actualiza el version de la respuesta local
							respuestasEvaluacion.get(preguntaActual).setVersion(result.getVersion());
						}
					});

				} else {
					SC.warn("No hay marcada la soluci�n");
				}

			}
		});

		btnGrabarNota.setSize("*", "40px");
		hLayout_1.addMember(btnGrabarNota);

		IButton btnSiguiente = new IButton("Siguiente");
		btnSiguiente.setSize("*", "50px");
		btnSiguiente.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				siguientePregunta();
			}
		});
		hLayout_1.addMember(btnSiguiente);
		layout.addMember(hLayout_1);
		hLayout.addMember(layout);

		listadoRespuestas.addRecordClickHandler(new RecordClickHandler() {
			
			@Override
			public void onRecordClick(RecordClickEvent event) {
				Record seleccionado = listadoRespuestas.getSelectedRecord();
				int n = listadoRespuestas.getRecordIndex(seleccionado);

				pintaRespuesta(n);
			}
		});

		setMargin(10);
		addItem(hLayout);
	}

	protected void pintaRespuesta(int nRespuesta) {
		this.preguntaActual = nRespuesta;
		RespuestaAlumno respuesta = respuestasEvaluacion.get(nRespuesta);

		String tipoPregunta = respuesta.getEvaluacionPregunta().getPregunta().getTipoPregunta().getCodigo();

		if ("VF".equals(tipoPregunta)) {
			panelMuestraRespuesta = new PanelCorreccionVF();
		} else if ("OM".equals(tipoPregunta))
			panelMuestraRespuesta = new PanelCorreccionOM();
		else if ("RL".equals(tipoPregunta))
			panelMuestraRespuesta = new PanelCorreccionRL();
		else
			panelMuestraRespuesta = null;

		if (panelMuestraRespuesta != null) {
			panelMuestraRespuesta.setRespuestaAlumno(respuesta);

			// obtener la respuesta correcta
			preguntasService.obtenerRespuestasPregunta(respuesta.getEvaluacionPregunta().getPreguntaID(),
					new AsyncCallback<List<RespuestaPregunta>>() {

						@Override
						public void onSuccess(List<RespuestaPregunta> result) {
							panelMuestraRespuesta.setRespuestaCorrecta(result);
						}

						@Override
						public void onFailure(Throwable caught) {
							SC.warn("Error obteniendo la respuesta correcta");
						}
					});
		}

		if (canvas.getChildren() != null && canvas.getChildren().length > 0)
			canvas.removeChild(canvas.getChildren()[0]);

		radioGroupItem.clearValue();
		if (panelMuestraRespuesta != null) {
			canvas.addChild(panelMuestraRespuesta);

			lblEnunciado.setContents(respuesta.getEvaluacionPregunta().getPregunta().getEnunciado());
			
			Record seleccionado = listadoRespuestas.getRecord(nRespuesta);
			Integer nota = seleccionado.getAttributeAsInt("nota");
			if (nota != null) {
				radioGroupItem.setValue(nota.toString());
			} 
		}
	}

	public void defineRespuestas(List<RespuestaAlumno> respuestas) {
		this.preguntaActual = 0;
		this.respuestasEvaluacion = respuestas;

		listadoRespuestas.setData(new ListGridRecord[] {});

		for (RespuestaAlumno respuesta : respuestas) {

			ListGridRecord r = new ListGridRecord();
			r.setAttribute("id", respuesta.getId());
			r.setAttribute("titulo", respuesta.getEvaluacionPregunta().getPregunta().getTitulo());
			r.setAttribute("nota", respuesta.getNota());
			r.setAttribute("tipo", respuesta.getEvaluacionPregunta().getPregunta().getTipoPregunta().getCodigo());

			listadoRespuestas.addData(r);
		}

		pintaRespuesta(0);
		listadoRespuestas.selectRecord(0);
	}

	protected void siguientePregunta() {

		if ((preguntaActual + 1) >= respuestasEvaluacion.size())
			SC.warn("Ya es la última pregunta");
		else {
			listadoRespuestas.deselectAllRecords();

			pintaRespuesta(preguntaActual + 1);
		}

	}

	protected void anteriorPregunta() {

		if (preguntaActual < 1)
			SC.warn("Ya es la primera pregunta");
		else {
			listadoRespuestas.deselectAllRecords();
			pintaRespuesta(preguntaActual - 1);
		}
	}

	public void defineEvaluacionAlumnoID(Integer evaluacion) {
		this.evaluacion = evaluacion;
	}
}
