package es.upv.client.ui;

import java.util.List;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.smartgwt.client.types.Alignment;
import com.smartgwt.client.types.AnimationEffect;
import com.smartgwt.client.types.VerticalAlignment;
import com.smartgwt.client.util.SC;
import com.smartgwt.client.widgets.Canvas;
import com.smartgwt.client.widgets.EdgedCanvas;
import com.smartgwt.client.widgets.IButton;
import com.smartgwt.client.widgets.Label;
import com.smartgwt.client.widgets.Window;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.layout.HLayout;
import com.smartgwt.client.widgets.layout.VLayout;

import es.upv.client.ui.panelrespuesta.PanelRespuestaBase;
import es.upv.client.ui.panelrespuesta.PanelRespuestaTipoOM;
import es.upv.client.ui.panelrespuesta.PanelRespuestaTipoRL;
import es.upv.client.ui.panelrespuesta.PanelRespuestaTipoVF;
import es.upv.shared.dto.Pregunta;
import es.upv.shared.dto.RespuestaAlumno;
import es.upv.shared.dto.RespuestaPregunta;
import es.upv.shared.services.EvaluacionServiceAsync;
import es.upv.shared.services.PreguntasServiceAsync;

public class PanelRespuesta extends Window {

	private static final String PREGUNTA_N_LABEL = "Pregunta nº";
	private Label lblEnunciado;
	private Label lblPreguntaTitulo;

	private IButton btnResponder;
	private Label lblPreguntaSinGuardar;

	/**
	 * Las preguntas que este panel gestionar�
	 */
	private List<RespuestaAlumno> preguntasEvaluacion;
	private int preguntaActual = 0;

	private PanelRespuestaBase panelRespuestaActual;
	private Canvas panelRespuesta;

	private EvaluacionServiceAsync evaluacionService = EvaluacionServiceAsync.Util.getInstance();
	private PreguntasServiceAsync preguntaService = PreguntasServiceAsync.Util.getInstance();

	public PanelRespuesta() {

		setAutoCenter(true);
		setShowMinimizeButton(false);
		setShowHeader(true);
		setShowEdges(false);
		setShowResizeBar(true);
		setShowTitle(true);
		setShowHeaderIcon(false);
		setShowModalMask(true);
		setAnimateShowEffect(AnimationEffect.FADE);
		setModalMaskOpacity(30);
		setIsModal(true);
		setShowShadow(true);

		HLayout hLayout = new HLayout();
		// hLayout.setHeight("420px");
		// hLayout.setWidth("80%");
		// hLayout.setAutoWidth();
		hLayout.setMembersMargin(30);
		hLayout.setPadding(10);

		setBackgroundColor("#eee");
		setBorder("10px solid #eee");

		IButton botonAnterior = new IButton("<strong>Anterior</strong>");
		botonAnterior.setAlign(Alignment.CENTER);
		botonAnterior.setLayoutAlign(VerticalAlignment.CENTER);
		botonAnterior.setWidth(140);
		botonAnterior.setHeight(50);
		botonAnterior.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {
				anteriorPregunta();
			}
		});

		hLayout.addMember(botonAnterior);

		VLayout columnaCentral = new VLayout();
		columnaCentral.setMembersMargin(10);
		columnaCentral.setWidth("*");

		lblPreguntaTitulo = new Label(PREGUNTA_N_LABEL);
		lblPreguntaTitulo.setHeight("30px");
		columnaCentral.addMember(lblPreguntaTitulo);

		lblEnunciado = new Label("");
		lblEnunciado.setHeight(60);
		lblEnunciado.setAutoFit(true);
		lblEnunciado.setWidth100();
		columnaCentral.addMember(lblEnunciado);

		EdgedCanvas edgedCanvas = new EdgedCanvas();
		edgedCanvas.setHeight("*");

		VLayout vStack = new VLayout();
		vStack.setPadding(5);
		vStack.setHeight(100);

		Label lblRespuesta = new Label("<h2>Respuesta</h2>");
		vStack.addMember(lblRespuesta);
		lblRespuesta.setHeight(20);
		lblRespuesta.setAlign(Alignment.CENTER);

		panelRespuesta = new Canvas();
		panelRespuesta.setPadding(10);
		panelRespuesta.setTop(10);
		panelRespuesta.setWidth100();
		panelRespuesta.setHeight("*");

		vStack.addMember(panelRespuesta);

		btnResponder = new IButton("Responder");
		btnResponder.setLayoutAlign(Alignment.RIGHT);
		btnResponder.setDisabled(true);
		// layout.addMember(btnResponder);
		btnResponder.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {
				respondePreguntaActual();
			}
		});
		vStack.addMember(btnResponder);

		lblPreguntaSinGuardar = new Label("");
		lblPreguntaSinGuardar.setHeight("32px");
		// lblPreguntaSinGuardar.setIcon("[SKIN]/actions/help.png");
		lblPreguntaSinGuardar.setAlign(Alignment.RIGHT);
		vStack.addMember(lblPreguntaSinGuardar);

		edgedCanvas.addChild(vStack);
		columnaCentral.addMember(edgedCanvas);

		IButton botonSiguiente = new IButton("<strong>Siguiente</strong>");
		botonSiguiente.setAlign(Alignment.CENTER);
		botonSiguiente.setLayoutAlign(VerticalAlignment.CENTER);
		botonSiguiente.setWidth(140);
		botonSiguiente.setHeight(50);

		botonSiguiente.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {
				siguientePregunta();
			}
		});

		hLayout.addMember(columnaCentral);
		hLayout.addMember(botonSiguiente);

		addItem(hLayout);
	}

	public void definePreguntas(List<RespuestaAlumno> preguntas) {
		this.preguntaActual = 0;
		this.preguntasEvaluacion = preguntas;
	}

	public void comienzaEvaluacion() {
		comienzaEvaluacion(0);
	}

	public void comienzaEvaluacion(int npreguntaInicio) {
		if (this.preguntasEvaluacion == null) {
			SC.say("No existen preguntas");
			return;
		}

		if (npreguntaInicio < 0 || npreguntaInicio >= preguntasEvaluacion.size())
			SC.say("Pregunta incorrecta");
		else
			pintaPregunta(npreguntaInicio);
	}

	protected void pintaPregunta(int nPregunta) {
		preguntaActual = nPregunta;
		Log.debug("preguntaActual es " + preguntaActual);
		final RespuestaAlumno respuestaActual = preguntasEvaluacion.get(preguntaActual);

		Pregunta pregunta = respuestaActual.getEvaluacionPregunta().getPregunta();

		Log.debug("tipo preguntaActual es " + pregunta.getTipoPreguntaID());

		lblEnunciado.setContents(pregunta.getEnunciado());
		lblPreguntaTitulo.setContents(PREGUNTA_N_LABEL + nPregunta + " : " + pregunta.getTitulo());

		lblPreguntaSinGuardar.setContents("Pregunta sin guardar");
		lblPreguntaSinGuardar.setIcon("[SKIN]/actions/help.png");

		// define el panel de respuesta seg�n el tipo de pregunta
		if (panelRespuesta.getChildren() != null && panelRespuesta.getChildren().length > 0)
			panelRespuesta.removeChild(panelRespuesta.getChildren()[0]);

		if ("OM".equals(pregunta.getTipoPregunta().getCodigo())) {
			
			AsyncCallback<List<RespuestaPregunta>> callback = new AsyncCallback<List<RespuestaPregunta>>() {
				@Override
				public void onSuccess(List<RespuestaPregunta> opciones) {

					PanelRespuestaBase children = new PanelRespuestaTipoOM(opciones);
					children.setRespuesta(respuestaActual.getOm());
					children.setWidth100();
					children.setHeight100();
					
					if (respuestaActual.getOm() != null) 
						lblPreguntaSinGuardar.setContents("Pregunta ya contestada");
					

					panelRespuesta.addChild(children);
					panelRespuestaActual = (PanelRespuestaBase) children;
				}

				@Override
				public void onFailure(Throwable caught) {
					SC.warn("Error al obtener las respuestas posibles");

				}
			};
			
			// obtener las opciones para esta pregunta antes de mostrar el
			// component
			Log.debug("obteniendo respuestas de pregunta " + pregunta.getId());
			preguntaService.obtenerRespuestasPregunta(pregunta.getId(), callback);

		} else {

			PanelRespuestaBase children = null;
			if ("VF".equals(pregunta.getTipoPregunta().getCodigo())) {
				children = new PanelRespuestaTipoVF(pregunta);
				children.setRespuesta(respuestaActual.getVf());
				children.setWidth100();
				
				if (respuestaActual.getOm() != null) 
					lblPreguntaSinGuardar.setContents("Pregunta ya contestada");
				
			} else if ("RL".equals(pregunta.getTipoPregunta().getCodigo())) {

				children = new PanelRespuestaTipoRL(null);
				children.setRespuesta(respuestaActual.getRl());
				children.setWidth100();
				children.setHeight100();
				
				if (respuestaActual.getOm() != null) 
					lblPreguntaSinGuardar.setContents("Pregunta ya contestada");

			}

			if (children != null) {
				panelRespuesta.addChild(children);
				panelRespuestaActual = children;			}

		}

		btnResponder.setDisabled(false);
	}

	protected void siguientePregunta() {
		if (preguntaActual == (preguntasEvaluacion.size() - 1)) {
			SC.say("Ya es la última pregunta!");
			return;
		}

		pintaPregunta(preguntaActual + 1);

	}

	protected void anteriorPregunta() {

		if (preguntaActual == 0) {
			SC.say("Ya está en la primera pregunta!");
			return;
		}

		pintaPregunta(preguntaActual - 1);
	}

	protected void respondePreguntaActual() {

		if (!panelRespuestaActual.contieneRespuesta()) {
			// valida que exista un valor
			SC.say("Debe escoger una opcion");
			return;
		}

		btnResponder.setDisabled(true);

		RespuestaAlumno respuestaActual = preguntasEvaluacion.get(preguntaActual);

		panelRespuestaActual.actualizaRespuesta(respuestaActual);

		evaluacionService.guardaRespuesta(respuestaActual.getEvaluacionAlumnoID(), respuestaActual,
				new AsyncCallback<RespuestaAlumno>() {

					@Override
					public void onSuccess(RespuestaAlumno result) {
						lblPreguntaSinGuardar.setContents("Respuesta guardada");
						lblPreguntaSinGuardar.setIcon("[SKIN]/actions/ok.png");
						
						preguntasEvaluacion.get(preguntaActual).setVersion(result.getVersion());
					}

					@Override
					public void onFailure(Throwable caught) {
						lblPreguntaSinGuardar.setContents("Error al guardar la respuesta");
						lblPreguntaSinGuardar.setIcon("[SKIN]/actions/exclamation.png");
					}
				});

	}
}
