package es.upv.client.ui;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.smartgwt.client.data.Criteria;
import com.smartgwt.client.data.DSCallback;
import com.smartgwt.client.data.DSRequest;
import com.smartgwt.client.data.DSResponse;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.types.Alignment;
import com.smartgwt.client.types.ExpansionMode;
import com.smartgwt.client.types.Overflow;
import com.smartgwt.client.util.BooleanCallback;
import com.smartgwt.client.util.SC;
import com.smartgwt.client.widgets.Canvas;
import com.smartgwt.client.widgets.EdgedCanvas;
import com.smartgwt.client.widgets.IButton;
import com.smartgwt.client.widgets.Label;
import com.smartgwt.client.widgets.events.CloseClickHandler;
import com.smartgwt.client.widgets.events.CloseClientEvent;
import com.smartgwt.client.widgets.form.DynamicForm;
import com.smartgwt.client.widgets.form.fields.FormItemIcon;
import com.smartgwt.client.widgets.form.fields.TextItem;
import com.smartgwt.client.widgets.form.fields.events.ChangedEvent;
import com.smartgwt.client.widgets.form.fields.events.ChangedHandler;
import com.smartgwt.client.widgets.form.fields.events.ClickEvent;
import com.smartgwt.client.widgets.form.fields.events.ClickHandler;
import com.smartgwt.client.widgets.form.fields.events.IconClickEvent;
import com.smartgwt.client.widgets.form.fields.events.IconClickHandler;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridField;
import com.smartgwt.client.widgets.grid.events.RecordClickEvent;
import com.smartgwt.client.widgets.grid.events.RecordClickHandler;
import com.smartgwt.client.widgets.layout.HLayout;
import com.smartgwt.client.widgets.layout.VLayout;
import com.smartgwt.client.widgets.tab.Tab;
import com.smartgwt.client.widgets.tab.TabSet;
import com.smartgwt.client.widgets.tab.events.TabDeselectedEvent;
import com.smartgwt.client.widgets.tab.events.TabDeselectedHandler;
import com.smartgwt.client.widgets.tab.events.TabSelectedEvent;
import com.smartgwt.client.widgets.tab.events.TabSelectedHandler;

import es.upv.client.ui.ds.PreguntasDatasource;
import es.upv.client.ui.ds.TipoPreguntaDatasource;
import es.upv.client.ui.events.GuardarRespuestasEvent;
import es.upv.client.ui.events.GuardarRespuestasEventHandler;
import es.upv.client.ui.respuestas.PanelDefinicionRespuestasBase;
import es.upv.client.ui.respuestas.PanelDefinicionRespuestasOM;
import es.upv.client.ui.respuestas.PanelDefinicionRespuestasRL;
import es.upv.client.ui.respuestas.PanelDefinicionRespuestasVF;
import es.upv.shared.dto.RespuestaPregunta;
import es.upv.shared.services.PreguntasServiceAsync;

public class PreguntasComponent extends com.smartgwt.client.widgets.layout.VLayout {

	/**
	 * Create a remote service proxy to talk to the server-side Greeting
	 * service.
	 */

	private ListGrid listadoPreguntas;
	private PreguntasDatasource preguntasDS;
	private DynamicForm preguntaForm;
	private Tab tabRespuestas;
	private TabSet tabSet;

	private PanelDefinicionRespuestasBase panelRespuestaActual;

	private Map<String, PanelDefinicionRespuestasBase> definicionRespuestas;
	private Canvas panelRespuestasDesconocido;
	private Canvas panelRespuestasError;
	private Canvas panelRespuestasSinGuardar;

	private TiposPreguntasDialog tiposDialog = null;
	private TipoPreguntaDatasource tiposDS = new TipoPreguntaDatasource();

	private PreguntasServiceAsync preguntaService = PreguntasServiceAsync.Util.getInstance();

	public PreguntasComponent() {

		final TemasDialog d = new TemasDialog();

		// resetea el dialogo de temas al cerrarlo
		d.addCloseClickHandler(new CloseClickHandler() {
			@Override
			public void onCloseClick(CloseClientEvent event) {
				d.reseteaDialogo();
				d.hide();
			}
		});

		final TemaAsignaturaSelector temaAsignaturaSelector = new TemaAsignaturaSelector(true);
		temaAsignaturaSelector.setWidth(600);

		temaAsignaturaSelector.setMostrarEditarTemasHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {

				if (temaAsignaturaSelector.getAsignaturaSeleccionada() != null) {
					d.show();
					d.setAsignatura(temaAsignaturaSelector.getAsignaturaSeleccionada());
				} else {
					SC.say("Seleccione un asignatura primero");
				}
			}
		});

		temaAsignaturaSelector.setLayoutAlign(Alignment.CENTER);
		addMember(temaAsignaturaSelector);

		Label lblPreguntas = new Label("<strong>Preguntas</strong>");
		lblPreguntas.setHeight("24px");
		addMember(lblPreguntas);

		preguntasDS = new PreguntasDatasource();

		EdgedCanvas edgedCanvas = new EdgedCanvas();
		edgedCanvas.setOverflow(Overflow.VISIBLE);

		HLayout hLayout = new HLayout();
		hLayout.setPadding(5);
		hLayout.setOverflow(Overflow.VISIBLE);
		hLayout.setMembersMargin(10);

		temaAsignaturaSelector.setTemaSeleccionadoHandler(new ChangedHandler() {
			public void onChanged(ChangedEvent event) {

				// mostrar preguntas de un tema en el listGrid
				// listadoPreguntas.setData(new ListGridRecord[] {} );

				Criteria criteria = new Criteria();
				criteria.addCriteria("tema", temaAsignaturaSelector.getTemaSeleccionado());
				listadoPreguntas.fetchData(criteria);

				preguntaForm.setValue("temaseleccionado", temaAsignaturaSelector.getTemaSeleccionado());
			}
		});

		listadoPreguntas = new ListGrid();
		listadoPreguntas.setWidth("45%");

		// expandir y mostar el enunciado
		listadoPreguntas.setCanExpandRecords(true);
		listadoPreguntas.setExpansionMode(ExpansionMode.DETAIL_FIELD);
		listadoPreguntas.setDetailField("enunciado");

		ListGridField fId = new ListGridField("id");
		fId.setHidden(true);
		ListGridField fNombre = new ListGridField("titulo", "Título");
		fNombre.setWidth("*");
		ListGridField fCodigo = new ListGridField("codigo", "Código");
		fCodigo.setWidth("10%");
		ListGridField fEnunciado = new ListGridField("enunciado", "Enunciado");
		fEnunciado.setHidden(true);
		ListGridField fTipo = new ListGridField("tipoPreguntaNombre", "Tipo");
		fTipo.setWidth("20%");

		listadoPreguntas.setFields(fId, fNombre, fCodigo, fTipo, fEnunciado);
		listadoPreguntas.setDataSource(preguntasDS);

		listadoPreguntas.addRecordClickHandler(new RecordClickHandler() {

			@Override
			public void onRecordClick(RecordClickEvent event) {
				tabSet.selectTab(0);
				preguntaForm.reset();
				preguntaForm.editSelectedData(listadoPreguntas);

				preguntaForm.setValue("temaseleccionado", temaAsignaturaSelector.getTemaSeleccionado());

				// preguntaForm.editNewRecord();
			}
		});

		hLayout.addMember(listadoPreguntas);

		tabSet = new TabSet();
		tabSet.setMinHeight(700);

		Tab tabPregunta = new Tab("Datos pregunta");

		preguntaForm = new DynamicForm();
		preguntaForm.setColWidths(60, "*");

		preguntaForm.setDataSource(preguntasDS);
		preguntaForm.setCanSubmit(false);

		preguntaForm.getField("titulo").setWidth("*");

		preguntaForm.getField("enunciado").setWidth("*");
		preguntaForm.getField("palabrasClave").setWidth("*");
		preguntaForm.getField("palabrasClave").setHeight(35);
		((TextItem) preguntaForm.getField("dificultad")).setKeyPressFilter("[0-9.]");

		preguntaForm.getField("tipoPreguntaID").setEndRow(false);
		preguntaForm.getField("tipoPreguntaID").setOptionDataSource(tiposDS);
		preguntaForm.getField("tipoPreguntaID").setValueField("id");
		preguntaForm.getField("tipoPreguntaID").setDisplayField("nombre");

		FormItemIcon editTiposIcon = new FormItemIcon();
		editTiposIcon.setName("editTiposIcon");
		editTiposIcon.setSrc("[SKIN]/actions/edit.png");

		preguntaForm.getField("tipoPreguntaID").setHint("Para editar los tipos de pregunta pulse el icono");
		preguntaForm.getField("tipoPreguntaID").setIcons(editTiposIcon);
		preguntaForm.getField("tipoPreguntaID").setShowIcons(true);
		preguntaForm.getField("tipoPreguntaID").addIconClickHandler(new IconClickHandler() {
			public void onIconClick(IconClickEvent event) {
				Log.debug("IconClick!!" + event.getIcon().getName());

				// el evento se lanza tanto para el picker como para el icono de
				// edicion
				// ejecutar la acción sólo en el icono de edición
				if ("editTiposIcon".equals(event.getIcon().getName())) {
					if (tiposDialog == null)
						tiposDialog = new TiposPreguntasDialog(tiposDS);

					tiposDialog.show();
				}
			}
		});

		// *************
		// acciones de preugnta
		IButton button = new IButton("Guardar pregunta");
		button.setAutoFit(true);
		button.setLayoutAlign(Alignment.CENTER);
		button.setIcon("[SKIN]/actions/save.png");
		button.addClickHandler(new com.smartgwt.client.widgets.events.ClickHandler() {

			@Override
			public void onClick(com.smartgwt.client.widgets.events.ClickEvent event) {
				final DSCallback call = new DSCallback() {

					@Override
					public void execute(DSResponse response, Object rawData, DSRequest request) {

						Log.debug("invalidando cache");
						Log.debug("n registros guardados " + response.getData().length);

						listadoPreguntas.invalidateCache();
						Criteria criteria = new Criteria();
						criteria.addCriteria("tema", temaAsignaturaSelector.getTemaSeleccionado());
						listadoPreguntas.fetchData(criteria);
					}
				};

				if (preguntaForm.validate()) {
					// si no es una pregunta nueva, comprobar si el tipo de
					// pregunta ha cambiado.
					final String preguntaID = preguntaForm.getValueAsString("id");
					if (preguntaID != null
							&& !preguntaForm.getValue("tipoPreguntaID").equals(
									preguntaForm.getValue("tipoPreguntaIDOriginal")))

						SC.confirm("El tipo ha cambiado, al guardar se eliminará la(s) respuesta(s) asociada(s)",
								new BooleanCallback() {

									@Override
									public void execute(Boolean value) {

										if (value) {
											// eliminar las respuestas asociadas
											// a esta pregunta
											preguntaService.eliminarRespuestasPregunta(new Long(preguntaID),
													new AsyncCallback<Void>() {
														@Override
														public void onFailure(Throwable caught) {
															SC.warn("No se ha podido eliminar las respuestas. "
																	+ "No se puede continuar");
														}

														public void onSuccess(Void result) {
															preguntaForm.saveData(call);
														};
													});
										}
									}
								});
					else {
						preguntaForm.saveData(call);
						preguntaForm.rememberValues();
					}

				}
			}
		});

		IButton btnNuevaPregunta = new IButton("Nueva pregunta");
		btnNuevaPregunta.setIcon("[SKIN]/actions/add.png");
		btnNuevaPregunta.setAutoFit(true);
		btnNuevaPregunta.setLayoutAlign(Alignment.CENTER);
		btnNuevaPregunta.addClickHandler(new com.smartgwt.client.widgets.events.ClickHandler() {

			@Override
			public void onClick(com.smartgwt.client.widgets.events.ClickEvent event) {
				preguntaForm.editNewRecord();

				if (temaAsignaturaSelector.getTemaSeleccionado() != null)
					preguntaForm.setValue("tema", temaAsignaturaSelector.getTemaSeleccionado());
			}
		});

		IButton button3 = new IButton("Elimina pregunta");
		button3.setIcon("[SKIN]/actions/remove.png");
		button3.setAutoFit(true);
		button3.setLayoutAlign(Alignment.CENTER);
		button3.addClickHandler(new com.smartgwt.client.widgets.events.ClickHandler() {

			@Override
			public void onClick(com.smartgwt.client.widgets.events.ClickEvent event) {
				listadoPreguntas.removeSelectedData(new DSCallback() {

					@Override
					public void execute(DSResponse response, Object rawData, DSRequest request) {
						preguntaForm.clearValue("version");
						preguntaForm.clearValue("id");
						preguntaForm.clearValue("tema");
					}
				}, new DSRequest());
			}
		});
		VLayout formLayout = new VLayout();
		formLayout.setMinHeight(700);
		formLayout.setOverflow(Overflow.VISIBLE);
		formLayout.setWidth("92%");
		formLayout.setMembersMargin(5);

		formLayout.addMember(preguntaForm);
		formLayout.addMember(button);
		formLayout.addMember(btnNuevaPregunta);
		formLayout.addMember(button3);

		tabPregunta.setPane(formLayout);
		tabSet.addTab(tabPregunta);

		tabRespuestas = new Tab("Respuesta(s)");

		registraComponentesRespuesta();

		tabRespuestas.setPane(obtenerComponenteRespuesta());

		// para evitar un flickeo al cambiar el tipo
		tabRespuestas.addTabDeselectedHandler(new TabDeselectedHandler() {

			@Override
			public void onTabDeselected(TabDeselectedEvent event) {
				tabRespuestas.getPane().hide();
			}
		});

		// cambia el componente de definición al seleccionar el tab
		tabRespuestas.addTabSelectedHandler(new TabSelectedHandler() {

			@Override
			public void onTabSelected(TabSelectedEvent event) {

				final Canvas panelRespuesta = obtenerComponenteRespuesta();

				if (panelRespuesta instanceof PanelDefinicionRespuestasBase) {

					// cargar las respuestas sólo si no es nuevo
					if (preguntaForm.getValue("id") != null) {
						Log.debug(("obteniendo respuestas para " + preguntaForm.getValueAsString("id")));

						// cargar las respuestas de la pregunta
						preguntaService.obtenerRespuestasPregunta(new Long(preguntaForm.getValueAsString("id")),
								new AsyncCallback<List<RespuestaPregunta>>() {

									@Override
									public void onSuccess(List<RespuestaPregunta> result) {
										panelRespuestaActual = (PanelDefinicionRespuestasBase) panelRespuesta;

										panelRespuestaActual.cargarRespuestas(result);

										Log.debug("Definiendo el tab con tipo:" + panelRespuestaActual.getID());
										tabSet.updateTab(tabRespuestas, panelRespuestaActual);
										tabRespuestas.getPane().show();
									}

									@Override
									public void onFailure(Throwable caught) {
										tabRespuestas.setPane(panelRespuestasError);
									}
								});
					} else
						tabRespuestas.setPane(panelRespuesta);
				} else
					tabRespuestas.setPane(panelRespuesta);
			}
		});

		tabSet.addTab(tabRespuestas);
		tabSet.setOverflow(Overflow.VISIBLE);
		hLayout.addMember(tabSet);

		// cuando el componente de respuestas diga que hay que guardar
		Root.EVENT_BUS.addHandler(GuardarRespuestasEvent.TYPE, new GuardarRespuestasEventHandler() {

			@Override
			public void onGuardarRespuestas(GuardarRespuestasEvent event) {
				if (preguntaForm.getValue("id") == null) {
					// pregunta no se ha guardado
					SC.warn("Es necesario guardar la pregunta antes de guardar las respuestas");
				} else {
					List<RespuestaPregunta> respuestas = event.getRespuestas();

					preguntaService.guardarRespuestasPregunta(new Long(preguntaForm.getValueAsString("id")),
							respuestas, new AsyncCallback<List<RespuestaPregunta>>() {
								public void onFailure(Throwable caught) {
									SC.warn("Error al guardar las respuestas");
								};

								public void onSuccess(java.util.List<RespuestaPregunta> result) {
									// cargar las respuestas en el panel de
									// definicion de respuestas
									panelRespuestaActual.cargarRespuestas(result);

								};
							});
				}
			}
		});

		edgedCanvas.setMinHeight(700);
		edgedCanvas.setOverflow(Overflow.VISIBLE);
		edgedCanvas.setCanDragResize(true);
		edgedCanvas.addChild(hLayout);
		addMember(edgedCanvas);
	}

	public void registraComponentesRespuesta() {

		definicionRespuestas = new HashMap<String, PanelDefinicionRespuestasBase>(3);

		registraTipo(new PanelDefinicionRespuestasOM());
		registraTipo(new PanelDefinicionRespuestasRL());
		registraTipo(new PanelDefinicionRespuestasVF());

		panelRespuestasDesconocido = new Canvas();
		panelRespuestasDesconocido.setWidth(300);
		Label lbl = new Label("<em><h4>Tipo de pregunta desconocido </h4></em>");
		lbl.setWidth100();
		lbl.setIcon("[SKIN]/Dialog/warn.png");
		panelRespuestasDesconocido.addChild(lbl);

		panelRespuestasError = new Canvas();
		panelRespuestasError.setWidth(300);
		Label lbl2 = new Label("<em><h4> Error al obtener los datos de las respuestas </h4></em>");
		lbl2.setWidth100();
		lbl2.setIcon("[SKIN]/Dialog/warn.png");
		panelRespuestasError.addChild(lbl2);

		panelRespuestasSinGuardar = new Canvas();
		panelRespuestasSinGuardar.setWidth(300);
		Label lbl3 = new Label("<em><h4> No se permite elegir respuestas de una pregunta nueva sin guardar o "
				+ "de una pregunta que haya cambiado el tipo. Debe guardar la pregunta. </h4></em>");
		lbl3.setWidth100();
		lbl3.setIcon("[SKIN]/Dialog/warn.png");
		panelRespuestasSinGuardar.addChild(lbl3);
	}

	private void registraTipo(PanelDefinicionRespuestasBase panel) {
		definicionRespuestas.put(panel.identificadorTipo(), panel);
	}

	/**
	 * @param codigoTipoPregunta
	 * @return El componente para el tipo de respueta o uno con mensaje de error
	 *         si no existe o es nulo
	 */
	public Canvas obtenerComponenteRespuesta() {

		// si es una pregunta nueva, no guardada, no permitir elegir respuestas
		final String preguntaID = preguntaForm.getValueAsString("id");

		Log.debug("tipoPreguntaOriginal", preguntaForm.getValueAsString("tipoPreguntaIDOriginal"));

		if (preguntaID == null
				|| !preguntaForm.getValue("tipoPreguntaID").equals(preguntaForm.getValue("tipoPreguntaIDOriginal")))
			return panelRespuestasSinGuardar;

		Record tipoSeleccionado = preguntaForm.getField("tipoPreguntaID").getSelectedRecord();
		Log.debug(("tipoSeleccionado " + tipoSeleccionado != null ? tipoSeleccionado.getAttribute("codigo") : null));

		String codigoTipoPregunta = tipoSeleccionado != null ? tipoSeleccionado.getAttribute("codigo") : null;

		if (codigoTipoPregunta != null && definicionRespuestas.containsKey(codigoTipoPregunta))
			return definicionRespuestas.get(codigoTipoPregunta);
		else
			return panelRespuestasDesconocido;
	}
}
