package es.upv.client.ui;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.activity.shared.ActivityManager;
import com.google.gwt.activity.shared.ActivityMapper;
import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Element;
import com.google.gwt.place.shared.Place;
import com.google.gwt.place.shared.PlaceController;
import com.google.gwt.place.shared.PlaceHistoryHandler;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.web.bindery.event.shared.EventBus;
import com.smartgwt.client.util.SC;

import es.upv.client.ClientFactory;
import es.upv.client.mvp.AppActivityMapper;
import es.upv.client.mvp.AppPlaceHistoryMapper;
import es.upv.client.place.DummyPanelPlace;
import es.upv.shared.services.SesionServiceAsync;

public class Root implements EntryPoint {
	private SesionServiceAsync sesionService = SesionServiceAsync.Util.getInstance();

	public static EventBus EVENT_BUS;

	private RootOneWidget appWidget = new RootOneWidget();
	private Place defaultPlace = new DummyPanelPlace();
	
	private PlaceController placeController;
	private ClientFactory clientFactory;

	public void onModuleLoad() {
		// Create ClientFactory using deferred binding so we can replace with
		// different
		// impls in gwt.xml
		clientFactory = GWT.create(ClientFactory.class);
		EventBus eventBus = clientFactory.getEventBus();
		EVENT_BUS = eventBus;
		placeController = clientFactory.getPlaceController();

		// Start ActivityManager for the main widget with our ActivityMapper
		ActivityMapper activityMapper = new AppActivityMapper(clientFactory);
		ActivityManager activityManager = new ActivityManager(activityMapper, eventBus);
		activityManager.setDisplay(appWidget);

		// Start PlaceHistoryHandler with our PlaceHistoryMapper
		AppPlaceHistoryMapper historyMapper = GWT.create(AppPlaceHistoryMapper.class);

		PlaceHistoryHandler historyHandler = new PlaceHistoryHandler(historyMapper);
		historyHandler.register(placeController, eventBus, defaultPlace);

		RootPanel.get("content").add(appWidget);
		// Goes to place represented on URL or default place
		historyHandler.handleCurrentHistory();

		Element link = DOM.getElementById("disconectLink");
		setDisconectLink(link, this);

		Element infoLink = DOM.getElementById("informationLink");
		setInformationLink(infoLink);
		
	}

	private native void setInformationLink(Element infoLink) /*-{

		infoLink.onclick = function() {
			@com.smartgwt.client.util.SC::warn(Ljava/lang/String;)("Aplicaci&oacute;n Evaltics GWT &copy; 2012<br>Jaime Serrano Cartagena");
			return false;
		};

	}-*/;

	private native void setDisconectLink(Element link, Root panel) /*-{

		link.onclick = function() {
			panel.@es.upv.client.ui.Root::desconectarUsuario()();
			return false;
		};

	}-*/;

	protected boolean desconectarUsuario() {
		Window.alert("Desconectando...");
		Log.debug("desceonctando usuario");
		sesionService.desconectarUsuario(new AsyncCallback<Boolean>() {
			@Override
			public void onFailure(Throwable caught) {
				SC.warn("No se ha podido desconectar la sesin. Debe salir del navegador para cerrar");
			}

			@Override
			public void onSuccess(Boolean result) {
				Element disconLink = DOM.getElementById("disconectLink");
				disconLink.setInnerHTML("<em>(no conectado)</em>");
				
				Element nombreUsuario = DOM.getElementById("nombreUsuario");
				nombreUsuario.setInnerHTML("");
				
				clientFactory.getPanelAlumnos().limpiaAlumno();
				
				placeController.goTo(new DummyPanelPlace());
			}
		});

		return false;
	}

}
