package es.upv.client.ui;

import java.util.LinkedHashMap;
import java.util.List;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.widgets.form.fields.ButtonItem;
import com.smartgwt.client.widgets.form.fields.FormItem;
import com.smartgwt.client.widgets.form.fields.SelectItem;
import com.smartgwt.client.widgets.form.fields.events.ChangedEvent;
import com.smartgwt.client.widgets.form.fields.events.ChangedHandler;
import com.smartgwt.client.widgets.form.fields.events.ClickEvent;
import com.smartgwt.client.widgets.form.fields.events.ClickHandler;

import es.upv.shared.dto.TemaAsignatura;

public class TemaAsignaturaSelector extends AsignaturaSelector {

	private SelectItem temaSelect;
	private ChangedHandler temaSeleccionadoHandler;

	private ClickHandler mostrarEditarTemasHandler;

	private boolean opcionEditarTemas = false;

	public TemaAsignaturaSelector() {
		super(false);

		setDefaultAsignaturaSeleccionadaHandler();
	}

	public TemaAsignaturaSelector(boolean mostrarOpcionEditar) {
		this.opcionEditarTemas = mostrarOpcionEditar;

		initialize(400);

		setDefaultAsignaturaSeleccionadaHandler();
	}

	@Override
	protected List<FormItem> obtenerCamposFormulario(boolean permiteMatricular, int anchoSelects) {
		List<FormItem> fields = super.obtenerCamposFormulario(false, anchoSelects);

		// final DataSource temasDS = new TemaAsignaturaDatasource();
		temaSelect = new SelectItem("temaAsignatura", "Tema asignatura");
		temaSelect.setStartRow(true);
		temaSelect.setEndRow(false);
		temaSelect.setWidth("*");
		temaSelect.setWrapTitle(false);

		/*
		 * temaSelect.setOptionDataSource(temasDS);
		 * temaSelect.setDisplayField("nombre"); temaSelect.setValueField("id");
		 * temaSelect.setFilterLocally(true);
		 * temaSelect.setAddUnknownValues(false);
		 */

		temaSelect.addChangedHandler(new ChangedHandler() {

			@Override
			public void onChanged(ChangedEvent event) {
				temaSeleccionadoHandler.onChanged(event);
			}
		});

		fields.add(temaSelect);

		if (opcionEditarTemas) {

			ButtonItem temasEdit = new ButtonItem("buttonEdit", "Editar temas asignatura");
			temasEdit.setIcon("[SKIN]/actions/edit.png");
			temasEdit.setStartRow(false);

			temasEdit.addClickHandler(new ClickHandler() {

				@Override
				public void onClick(ClickEvent event) {
					mostrarEditarTemasHandler.onClick(event);
				}
			});

			fields.add(temasEdit);
		}

		return fields;
	}

	public Record getTemaSeleccionadoRecord() {
		return temaSelect.getSelectedRecord();
	}

	public Integer getTemaSeleccionado() {
		if (temaSelect.getValue() != null)
			return new Integer(temaSelect.getValueAsString());
		else
			return null;
	}

	public ChangedHandler getTemaSeleccionadoHandler() {
		return temaSeleccionadoHandler;
	}

	public void setTemaSeleccionadoHandler(ChangedHandler temaSeleccionadoHandler) {
		this.temaSeleccionadoHandler = temaSeleccionadoHandler;
	}

	public ClickHandler getMostrarEditarTemasHandler() {
		return mostrarEditarTemasHandler;
	}

	public void setMostrarEditarTemasHandler(ClickHandler mostrarEditarTemasHandler) {
		this.mostrarEditarTemasHandler = mostrarEditarTemasHandler;
	}

	public void setDefaultAsignaturaSeleccionadaHandler() {
		this.setAsignaturaSeleccionadaHandler(new ChangedHandler() {

			@Override
			public void onChanged(ChangedEvent event) {

				Log.debug("Obteniendo temas de asignatura " + event.getValue().toString());
				// obtener los temas
				temaAsignaturaService.obtenerTemasAsignatura(new Long(event.getValue().toString()),
						new AsyncCallback<List<TemaAsignatura>>() {
							@Override
							public void onSuccess(List<TemaAsignatura> result) {
								LinkedHashMap<String, String> options = new LinkedHashMap<String, String>();

								if (result != null) {
									// definir las options
									for (TemaAsignatura tema : result) {
										options.put(tema.getId().toString(), tema.getNombre());
									}

								}

								// los temas validos para la asignatura
								// seleccionada
								temaSelect.setValueMap(options);

								// limpiar el selector al para evitar que queden
								// temas no disponibles seleccionados
								temaSelect.clearValue();
							};

							@Override
							public void onFailure(Throwable caught) {
								Log.debug("Error obteniendo temas");

							}
						});
			}
		});
	}
}
