package es.upv.client.ui;

import java.util.List;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.smartgwt.client.core.DataClass;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.types.Alignment;
import com.smartgwt.client.types.Autofit;
import com.smartgwt.client.types.ExpansionMode;
import com.smartgwt.client.util.SC;
import com.smartgwt.client.widgets.Dialog;
import com.smartgwt.client.widgets.IButton;
import com.smartgwt.client.widgets.Label;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.form.DynamicForm;
import com.smartgwt.client.widgets.form.events.SubmitValuesEvent;
import com.smartgwt.client.widgets.form.events.SubmitValuesHandler;
import com.smartgwt.client.widgets.form.fields.IntegerItem;
import com.smartgwt.client.widgets.form.fields.TextItem;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridField;
import com.smartgwt.client.widgets.grid.ListGridRecord;
import com.smartgwt.client.widgets.grid.events.RecordClickEvent;
import com.smartgwt.client.widgets.grid.events.RecordClickHandler;
import com.smartgwt.client.widgets.layout.HLayout;
import com.smartgwt.client.widgets.layout.VLayout;

import es.upv.client.ui.ds.TemaAsignaturaDatasource;
import es.upv.shared.dto.TemaAsignatura;
import es.upv.shared.model.AsignaturaOverlay;
import es.upv.shared.model.TemaAsignaturaOverlay;
import es.upv.shared.services.TemaAsignaturaServiceAsync;

public class TemasDialog extends Dialog {

	protected final TemaAsignaturaServiceAsync temaAsignaturaService = TemaAsignaturaServiceAsync.Util.getInstance();

	private TemaAsignaturaDatasource temaDS = new TemaAsignaturaDatasource();
	private ListGrid listadoTemas;

	private AsignaturaOverlay asignatura;
	private Label tituloAsignatura;

	public TemasDialog() {
		// setShowHeader(false);
		setCanDragResize(true);
		setCanDragReposition(true);
		
		setAutoHeight();
		this.setModalMaskOpacity(30);
		this.setShowModalMask(true);
		setTitle("Mantenimiento de temas");
		// resizeTo(350, 200);
		setHeight(500);
		setWidth(400);

		this.setIsModal(true);
		this.setCanDrag(true);

		VLayout layout = new VLayout(5);
		layout.setPadding(10);
		layout.setAutoHeight();
		
		listadoTemas = new ListGrid();
		listadoTemas.setCanExpandRecords(true);
		listadoTemas.setExpansionMode(ExpansionMode.DETAIL_FIELD);
		listadoTemas.setDetailField("descripcion");
		
		// listadoTemas.setMinHeight(120);
		listadoTemas.setHeight(120);
		listadoTemas.setAutoFitData(Autofit.VERTICAL);
		listadoTemas.setAutoFitMaxRecords(10);
		
		listadoTemas.setDataSource(temaDS);
		
		// ordenar por el numero de tema
		listadoTemas.setSortField("numero_tema");
		ListGridField fNombre = new ListGridField("nombre", "Nombre");
		fNombre.setWidth("*");
		ListGridField fTema = new ListGridField("numero_tema", "Nº Tema");
		fTema.setWidth("20%");
		
		listadoTemas.setFields(new ListGridField[] { fNombre, fTema });

		// accciones
		HLayout acciones = new HLayout(5);
		IButton eliminarBtn = new IButton("Eliminar");
		eliminarBtn.setLayoutAlign(Alignment.RIGHT);
		eliminarBtn.setIcon("[SKIN]/actions/remove.png");

		IButton nuevoBtn = new IButton("Nuevo tema");
		nuevoBtn.setLayoutAlign(Alignment.RIGHT);
		nuevoBtn.setIcon("[SKIN]/actions/add.png");

		acciones.addMember(eliminarBtn);
		acciones.addMember(nuevoBtn);

		final DynamicForm temaForm = new DynamicForm();
		temaForm.setWidth100();
		temaForm.setSaveOnEnter(true);
		temaForm.setDataSource(temaDS);
		
		temaForm.setCanSubmit(true);
		
		IntegerItem integerItem = new IntegerItem("numero_tema");
		integerItem.setTitle("Nº tema");
		
		temaForm.setDataSource(temaDS);
		temaForm.getField("nombre").setWidth("*");
		temaForm.getField("nombre").setHint("Nuevo tema");
		((TextItem) temaForm.getField("nombre")).setShowHintInField(true);
		temaForm.getField("descripcion").setWidth("*");

		listadoTemas.addRecordClickHandler(new RecordClickHandler() {

			@Override
			public void onRecordClick(RecordClickEvent event) {
				temaForm.reset();
				temaForm.editSelectedData(listadoTemas);
			}
		});

		temaForm.addSubmitValuesHandler(new SubmitValuesHandler() {

			@Override
			public void onSubmitValues(SubmitValuesEvent event) {
				if (! temaForm.validate()) 
					return;
				
				// jsc : temaForm.getValuesAsRecord devuelve los valores originales,
				
				TemaAsignaturaOverlay tema = event.getValues().cast();
				Log.debug("evnet data: " + (new Record(event.getValues()).getAttribute("nombre")));
				Log.debug(temaForm.getValueAsString ("nombre"));
				Log.debug(tema.getNombre());
				Log.debug("getValuesAsRecord " + temaForm.getValuesAsRecord().getAttribute("nombre"));
				
				int asigID = asignatura.getId();
				final TemaAsignatura temaDTO = new TemaAsignatura(null, tema.getNombre(), tema.getNumeroTema(), tema.getDescripcion(), new Long(asigID));

				if (tema.esNuevo()) {
					Log.debug("Guardando un nuevo tema");
				} else {
					int id = tema.getId();
					temaDTO.setId(new Long(id));
					temaDTO.setVersion(tema.getVersion());
					
					Log.debug(tema.getVersion() + "");
				}
				
				temaAsignaturaService.saveTema(temaDTO, new AsyncCallback<TemaAsignatura>() {
					@Override
					public void onSuccess(TemaAsignatura result) {
						
						// actualizar el version del formulario
						temaForm.setValue("version", result.getVersion());
						temaForm.setValue("id", result.getId().intValue());
						temaForm.saveData();
						
						if (temaDTO.getId() == null) temaForm.editNewRecord();
					}

					@Override
					public void onFailure(Throwable caught) {
						SC.warn("No se ha podido guardar el tema. No se pueden tener dos temas con el mismo número.");
					}
				});
			}
		});

		eliminarBtn.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {
				if (listadoTemas.getSelectedRecord() != null) {
					
					temaAsignaturaService.deleteTema(listadoTemas.getSelectedRecord().getAttributeAsInt("id"), new AsyncCallback<Void>() {
						@Override
						public void onFailure(Throwable caught) {
							SC.warn("Error al eliminar el tema");
						}
						
						@Override
						public void onSuccess(Void result) {
							listadoTemas.removeData(listadoTemas.getSelectedRecord());
							temaForm.clearValues();
						}
					});
					
				} else 
					SC.say("Ningún tema seleccionado");
			}
		});

		nuevoBtn.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {
				temaForm.resetValues();
				temaForm.editNewRecord();
				temaForm.focusInItem(temaForm.getField("nombre"));
				
				listadoTemas.deselectAllRecords();
			}
		});
		
		tituloAsignatura = new Label("");
		tituloAsignatura.setHeight(30);
		tituloAsignatura.setStyleName("tituloAsignatura");
		layout.addMember(tituloAsignatura);

		layout.addMember(listadoTemas);

		acciones.setAlign(Alignment.RIGHT);
		layout.addMember(acciones);
		layout.addMember(temaForm);
		
		IButton btnGuardar = new IButton("Guardar");
		btnGuardar.setIcon("[SKIN]/actions/save.png");
		btnGuardar.setLayoutAlign(Alignment.CENTER);
		
		btnGuardar.addClickHandler(new ClickHandler() {
			
			@Override
			public void onClick(ClickEvent event) {
				temaForm.submit();
			}
		});
		layout.addMember(btnGuardar);
		
		addItem(layout);
	}

	public void setAsignatura(AsignaturaOverlay asignatura) {

		this.asignatura = asignatura;

		listadoTemas.setData(new ListGridRecord[] {});
		temaDS.setTestData(new DataClass[] {});
		
		tituloAsignatura.setContents("Asignatura: <strong>" + asignatura.getNombre() + "</strong>");
		
		// obtener los temas
		temaAsignaturaService.obtenerTemasAsignatura(asignatura.getId(), new AsyncCallback<List<TemaAsignatura>>() {
			@Override
			public void onSuccess(List<TemaAsignatura> result) {

				if (result != null) {
					// definir las options
					for (TemaAsignatura tema : result) {
						ListGridRecord r = new ListGridRecord();
						r.setAttribute("id", tema.getId().intValue());
						r.setAttribute("numero_tema", tema.getNumero_tema());
						r.setAttribute("nombre", tema.getNombre());
						r.setAttribute("descripcion", tema.getDescripcion());
						r.setAttribute("version", tema.getVersion());

						listadoTemas.addData(r);
					}

					listadoTemas.fetchData();
				}

			};

			@Override
			public void onFailure(Throwable caught) {
				Log.debug("Error obteniendo temas");
			}
		});

	}
	
	public void reseteaDialogo ()  {
		listadoTemas.setData(new ListGridRecord[] {});
		temaDS.setTestData(new DataClass[] {});
		
		
		asignatura = null;
		tituloAsignatura.setContents("");
	}

}
