package es.upv.client.ui;

import com.smartgwt.client.types.Alignment;
import com.smartgwt.client.types.Autofit;
import com.smartgwt.client.util.SC;
import com.smartgwt.client.widgets.Dialog;
import com.smartgwt.client.widgets.IButton;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.form.DynamicForm;
import com.smartgwt.client.widgets.form.events.SubmitValuesEvent;
import com.smartgwt.client.widgets.form.events.SubmitValuesHandler;
import com.smartgwt.client.widgets.form.fields.FormItemIcon;
import com.smartgwt.client.widgets.form.fields.IntegerItem;
import com.smartgwt.client.widgets.form.fields.events.IconClickEvent;
import com.smartgwt.client.widgets.form.fields.events.IconClickHandler;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridField;
import com.smartgwt.client.widgets.grid.events.RecordClickEvent;
import com.smartgwt.client.widgets.grid.events.RecordClickHandler;
import com.smartgwt.client.widgets.layout.HLayout;
import com.smartgwt.client.widgets.layout.VLayout;

import es.upv.client.ui.ds.TipoPreguntaDatasource;
import es.upv.shared.services.TemaAsignaturaServiceAsync;

public class TiposPreguntasDialog extends Dialog {

	protected final TemaAsignaturaServiceAsync temaAsignaturaService = TemaAsignaturaServiceAsync.Util.getInstance();

	private TipoPreguntaDatasource tiposDS;
	private ListGrid listadoTipos;

	public TiposPreguntasDialog(TipoPreguntaDatasource tiposDS2) {
		tiposDS = tiposDS2;
		
		// setShowHeader(false);
		setCanDragResize(true);
		setCanDragReposition(true);
		
		setAutoHeight();
		this.setModalMaskOpacity(30);
		this.setShowModalMask(true);
		setTitle("Mantenimiento de tipos de pregunta");
		// resizeTo(350, 200);
		setHeight(500);
		setWidth(600);

		this.setIsModal(true);
		this.setCanDrag(true);

		VLayout layout = new VLayout(5);
		layout.setPadding(10);
		layout.setAutoHeight();
		
		listadoTipos = new ListGrid();
		
		listadoTipos.setHeight(120);
		listadoTipos.setAutoFitData(Autofit.VERTICAL);
		listadoTipos.setAutoFitMaxRecords(10);
		
		listadoTipos.setDataSource(tiposDS);
		listadoTipos.setAutoFetchData(true);
		
		// ordenar por el numero de tema
		ListGridField fNombre = new ListGridField("nombre", "Nombre");
		fNombre.setWidth("20%");
		ListGridField fDesc = new ListGridField("descripcion", "Descripción");
		fDesc.setWidth("*");
		ListGridField fCod = new ListGridField("codigo", "Código");
		fCod.setWidth("10%");
		
		listadoTipos.setFields(new ListGridField[] { fNombre, fCod, fDesc });

		// accciones
		HLayout acciones = new HLayout(5);
		IButton eliminarBtn = new IButton("Eliminar");
		eliminarBtn.setLayoutAlign(Alignment.RIGHT);
		eliminarBtn.setIcon("[SKIN]/actions/remove.png");

		IButton nuevoBtn = new IButton("Nuevo tema");
		nuevoBtn.setLayoutAlign(Alignment.RIGHT);
		nuevoBtn.setIcon("[SKIN]/actions/add.png");

		acciones.addMember(eliminarBtn);
		acciones.addMember(nuevoBtn);

		final DynamicForm tipoForm = new DynamicForm();
		tipoForm.setWidth100();
		tipoForm.setSaveOnEnter(true);
		tipoForm.setDataSource(tiposDS);
		
		tipoForm.setCanSubmit(true);
		
		IntegerItem integerItem = new IntegerItem("numero_tema");
		integerItem.setTitle("Nº tema");
		
		FormItemIcon icon = new FormItemIcon();  
        icon.setSrc("[SKIN]/actions/help.png");  
        
		final String textoAyuda = "<h3> Tipos de preguntas </h3><br/> Existen tres tipos de pregunta predefinidos que deben estar asociados a los siguientes códigos. <ul> " +
        		"<li> <strong> VF </strong> : Muestra una pregunta con opciones para eligir si la pregunta es verdadera o falsa</li> " +
        		"<li> <strong> OM </strong> : Muestra una pregunta con varias opciones para eligir la correcta </li> " +
        		"<li> <strong> RL </strong> : Muestra un cuadro para escribir la respuesta </li> " +
        		"</ul>";
		
		tipoForm.setDataSource(tiposDS);
		tipoForm.getField("codigo").setHint("El código controla los distintos tipos de opciones de pregunta");
		tipoForm.getField("codigo").setIcons(icon);
		tipoForm.getField("codigo").setIconPrompt(textoAyuda);
		tipoForm.getField("codigo").addIconClickHandler(new IconClickHandler() {  
            public void onIconClick(IconClickEvent event) {  
                SC.say(textoAyuda);  
            }  
        });  
		
		tipoForm.getField("nombre").setWidth("*");
		tipoForm.getField("descripcion").setWidth("*");

		listadoTipos.addRecordClickHandler(new RecordClickHandler() {

			@Override
			public void onRecordClick(RecordClickEvent event) {
				tipoForm.reset();
				tipoForm.editSelectedData(listadoTipos);
			}
		});

		tipoForm.addSubmitValuesHandler(new SubmitValuesHandler() {

			@Override
			public void onSubmitValues(SubmitValuesEvent event) {
				if (! tipoForm.validate()) 
					return;
				
				tipoForm.saveData();
			}
		});

		eliminarBtn.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {
				if (listadoTipos.getSelectedRecord() != null) {
					
					listadoTipos.removeSelectedData();
					
				} else 
					SC.say("Ningún tema seleccionado");
			}
		});

		nuevoBtn.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {
				tipoForm.resetValues();
				tipoForm.editNewRecord();
				tipoForm.focusInItem(tipoForm.getField("codigo"));
				listadoTipos.deselectAllRecords();
			}
		});
		
		layout.addMember(listadoTipos);

		acciones.setAlign(Alignment.RIGHT);
		layout.addMember(acciones);
		layout.addMember(tipoForm);
		
		IButton btnGuardar = new IButton("Guardar");
		btnGuardar.setIcon("[SKIN]/actions/save.png");
		btnGuardar.setLayoutAlign(Alignment.CENTER);
		
		btnGuardar.addClickHandler(new ClickHandler() {
			
			@Override
			public void onClick(ClickEvent event) {
				tipoForm.submit();
			}
		});
		layout.addMember(btnGuardar);
		
		addItem(layout);
	}
}
