package es.upv.client.ui.ds;

import java.util.List;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.smartgwt.client.data.Criteria;
import com.smartgwt.client.data.DSRequest;
import com.smartgwt.client.data.DSResponse;
import com.smartgwt.client.data.DataSourceField;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.data.fields.DataSourceDateField;
import com.smartgwt.client.data.fields.DataSourceIntegerField;
import com.smartgwt.client.types.DSDataFormat;
import com.smartgwt.client.types.FieldType;
import com.smartgwt.client.widgets.grid.ListGridRecord;

import es.upv.shared.dto.EvaluacionAlumno;
import es.upv.shared.services.EvaluacionServiceAsync;

public class EvaluacionAlumnoDatasource extends DatasourceEvaltics {

	private final static EvaluacionServiceAsync evaluacionService = EvaluacionServiceAsync.Util.getInstance();

	public EvaluacionAlumnoDatasource() {
		this.setDataFormat(DSDataFormat.JSON);

		DataSourceField fId = new DataSourceField("id", FieldType.INTEGER);
		fId.setPrimaryKey(true);
		fId.setHidden(true);
		addField(fId);

		DataSourceField fAsigID = new DataSourceField("asignaturaID",  FieldType.INTEGER);
		fAsigID.setRequired(true);
		fAsigID.setHidden(true);

		DataSourceField fEstado = new DataSourceField("estado", FieldType.TEXT);
		fEstado.setHidden(true);
		DataSourceField fAlumnoDNI = new DataSourceField("alumnoDNI", FieldType.TEXT);
		DataSourceField fAlumnoNombre = new DataSourceField("alumnoNombre", FieldType.TEXT);
		DataSourceField fFecha = new DataSourceDateField("fecha");
		
		DataSourceField fVersion = new DataSourceIntegerField("version");
		fVersion.setHidden(true);
		
		addField(fAsigID);
		addField(fEstado);
		addField(fAlumnoDNI);
		addField(fAlumnoNombre);
		addField(fFecha);
		addField(fVersion);
		
		setID("EvaluacionAlumnoDS");
		
	}

	@Override
	protected Object transformRequest(DSRequest request) {
		String requestId = request.getRequestId();
		DSResponse response = new DSResponse();
		response.setAttribute("clientContext", request.getAttributeAsObject("clientContext"));
		// Asume success
		response.setStatus(0);

		switch (request.getOperationType()) {
		case FETCH:
			executeFetch(requestId, response, request);
			break;
		case ADD:
			// executeAdd(requestId, request, response);
			break;
		case UPDATE:
			// executeUpdate(requestId, request, response);
			break;
		case REMOVE:
			// executeRemove(requestId, request, response);
			break;
		default:
			// Operation not implemented.
			break;
		}
		return request.getData();
	}

	private void executeFetch(final String requestId, final DSResponse response, final DSRequest request) {
		final Integer asignaturaId;
		Criteria crit = request.getCriteria();
		if (crit != null)
			asignaturaId = crit.getAttributeAsInt("asignatura_id");
		else
			asignaturaId = null;

		Log.debug("obteniendo evaluaciones de asignatura " + asignaturaId  + 
				 " : " + requestId);

		if (asignaturaId != null) {

			evaluacionService.obtenerEvaluacionesRealizadasAsignatura(new Long(asignaturaId),
					new AsyncCallback<List<EvaluacionAlumno>>() {

						@Override
						public void onFailure(Throwable caught) {
							// TODO Auto-generated method stub

						}

						@Override
						public void onSuccess(List<EvaluacionAlumno> result) {
							if (result != null) {

								Log.debug("N evaluaciones: " + result.size()   + 
										 " : " + requestId);
								
								// limpiar grid y aadir preguntas al GRID
								Record[] evaluaciones = new Record[result.size()];
								for (int i = 0; i < result.size(); i++) {
									

									evaluaciones[i] = new ListGridRecord();
									evaluaciones[i].setAttribute("id", result.get(i).getId().intValue());
									evaluaciones[i].setAttribute("estado", result.get(i));
									evaluaciones[i].setAttribute("asignaturaID", result.get(i).getAsignaturaID());
									evaluaciones[i].setAttribute("alumnoDNI", result.get(i).getAlumno().getDni());
									evaluaciones[i].setAttribute("alumnoNombre", result.get(i).getAlumno().getPrimerNombre() + " " + result.get(i).getAlumno().getApellidoPaterno());
									evaluaciones[i].setAttribute("version", result.get(i).getVersion());
									evaluaciones[i].setAttribute("fecha", result.get(i).getFecha());
								}

								response.setData(evaluaciones);
								response.setTotalRows(evaluaciones.length);
								processResponse(requestId, response);
							} else
								Window.alert("sin resultados");
						}

					});

		} else {
			response.setData(new Record[] {});
			response.setTotalRows(0);
			processResponse(requestId, response);
		}
	}

}
