package es.upv.client.ui.ds;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.smartgwt.client.data.Criteria;
import com.smartgwt.client.data.DSRequest;
import com.smartgwt.client.data.DSResponse;
import com.smartgwt.client.data.DataSource;
import com.smartgwt.client.data.DataSourceField;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.data.fields.DataSourceDateField;
import com.smartgwt.client.data.fields.DataSourceFloatField;
import com.smartgwt.client.data.fields.DataSourceIntegerField;
import com.smartgwt.client.data.fields.DataSourceTextField;
import com.smartgwt.client.types.DSDataFormat;
import com.smartgwt.client.types.DSProtocol;
import com.smartgwt.client.types.ValidatorType;
import com.smartgwt.client.util.SC;
import com.smartgwt.client.widgets.form.validator.Validator;
import com.smartgwt.client.widgets.grid.ListGridRecord;

import es.upv.shared.dto.Pregunta;
import es.upv.shared.services.TemaAsignaturaServiceAsync;

public class PreguntasDatasource extends DataSource {

	private TemaAsignaturaServiceAsync temaAsignaturaService = TemaAsignaturaServiceAsync.Util.getInstance();

	public PreguntasDatasource() {
		Validator required = new Validator();
		required.setType(ValidatorType.REQUIRED);

		DataSourceField f = new DataSourceIntegerField("id");
		f.setPrimaryKey(true);
		f.setHidden(true);
		DataSourceField fNombre = new DataSourceTextField("titulo", "Título", 128, true);
		fNombre.setValidators(required);
		DataSourceField fCodigo = new DataSourceTextField("codigo", "Código", 128, true);
		DataSourceField fEnun = new DataSourceTextField("enunciado", "Enunciado", 2000, true);

		addField(f);
		addField(fCodigo);
		addField(fNombre);
		addField(fEnun);
		
		DataSourceTextField fTipoPreguntaNombre = new DataSourceTextField ("tipoPreguntaNombre", "Tipo Pregunta", 20, true);
		fTipoPreguntaNombre.setHidden(true);
		addField(fTipoPreguntaNombre);
		
		DataSourceTextField fTipo = new DataSourceTextField("tipoPreguntaID", "Tipo pregunta", 20, true);
		Map<String, String> valueMap = new HashMap<String, String>();
		fTipo.setValueMap(valueMap);
		addField(fTipo);
		
		/* el campo tipo pregunta original se utiliza para comprobar si un tipo ha cambiado al guardar */
		DataSourceTextField fTipoOriginal = new DataSourceTextField("tipoPreguntaIDOriginal", "");
		fTipoOriginal.setHidden(true);
		addField (fTipoOriginal);

		DataSourceFloatField fDifi = new DataSourceFloatField("dificultad", "Dificultad", 10, true);
		addField(fDifi);

		DataSourceTextField fPC = new DataSourceTextField("palabrasClave", "Palabras clave", 1000, false);
		addField(fPC);

		DataSourceField fTema = new DataSourceIntegerField("tema");
		// fTema.setHidden(true);
		fTema.setCanEdit(false);
		addField(fTema);
		
		DataSourceField fTema2 = new DataSourceIntegerField("temaseleccionado");
		// fTema.setHidden(true);
		fTema2.setCanEdit(false);
		addField(fTema2);

		DataSourceField fVersion = new DataSourceIntegerField("version");
		fVersion.setHidden(true);
		addField(fVersion);
		
		DataSourceField fCreacion = new DataSourceDateField("fechaCreacion");
		fCreacion.setHidden(true);
		DataSourceField fModificacion = new DataSourceDateField("fechaModificacion");
		fModificacion.setHidden(true);
		
		addField(fCreacion);
		addField(fModificacion);

		setDataProtocol(DSProtocol.CLIENTCUSTOM);
		setDataFormat(DSDataFormat.CUSTOM);
		setClientOnly(false);
	}

	@Override
	protected Object transformRequest(DSRequest request) {
		String requestId = request.getRequestId();
		DSResponse response = new DSResponse();
		response.setAttribute("clientContext", request.getAttributeAsObject("clientContext"));
		// Asume success
		response.setStatus(0);

		switch (request.getOperationType()) {
		case FETCH:
			executeFetch(requestId, response, request);
			break;
		case ADD:
			executeAdd(requestId, request, response);
			break;
		case UPDATE:
			executeUpdate(requestId, request, response);
			break;
		case REMOVE:
			executeRemove(requestId, request, response);
			break;
		default:
			// Operation not implemented.
			break;
		}
		return request.getData();
	}

	private void executeUpdate(final String requestId, final DSRequest request, final DSResponse response) {
		Log.debug("update data");

		// para los daos, añadir y actualizar es lo mismo
		executeAdd(requestId, request, response);
	}

	private void executeAdd(final String requestId, final DSRequest request, final DSResponse response) {
		Log.debug("add data");

		// obtener el tema
		Record r = new Record(request.getData());
		String tema = r.getAttributeAsString("tema");
		Integer temaSeleccionado = r.getAttributeAsInt("temaseleccionado");
		
		// si no hay selccionada una pregunta, grabar el del ultimo tema seleccionado
		if (tema == null)
			tema = temaSeleccionado.toString();

		if (tema != null) {

			Pregunta p = new Pregunta();
			p.setTitulo(r.getAttribute("titulo"));
			p.setCodigo(r.getAttributeAsString("codigo"));
			
			if (r.getAttributeAsString("tipoPreguntaID") != null)
				p.setTipoPreguntaID(new Long(r.getAttributeAsString("tipoPreguntaID")));
			
			Date fCreacion = r.getAttributeAsDate("fechaCreacion");
			p.setFechaCreacion(fCreacion != null ? fCreacion : new Date());
			
			// actualizamos siempre la fecha de modificacion
			p.setFechaModificacion(new Date());
			
			p.setEnunciado(r.getAttributeAsString("enunciado"));
			p.setPalabrasClave(r.getAttributeAsString("palabrasClave"));
			p.setDificultad(r.getAttributeAsFloat("dificultad"));

			if (r.getAttribute("version") != null)
				p.setVersion(r.getAttributeAsInt("version"));

			if (r.getAttribute("id") != null)
				p.setId(Long.valueOf(r.getAttributeAsString("id")));

			// rellena datos de la pregunta guardada
			temaAsignaturaService.guardaPreguntaDeTema(new Long(tema), p, new AsyncCallback<Pregunta>() {
				public void onSuccess(Pregunta result) {
					// actualizar datos del listado

					Record r = new Record();
					r.setAttribute("id", result.getId());
					r.setAttribute("version", result.getVersion());
					r.setAttribute("tipoPreguntaIDOriginal", result.getTipoPreguntaID());

					response.setData(new Record[] { r });
					response.setInvalidateCache(true);
					processResponse(requestId, response);
				};

				@Override
				public void onFailure(Throwable caught) {
					Window.alert("Error al guardar");
				}
			});
		} else {
			Window.alert("Es necesario elegir un tema");
		}

	}

	private void executeRemove(final String requestId, final DSRequest request, final DSResponse response) {
		Log.debug("remove data");

		// obtener el tema
		Record r = new Record(request.getData());
		int id = r.getAttributeAsInt("id");
		int tema = r.getAttributeAsInt("tema");
		
		temaAsignaturaService.eliminaPreguntaTema(new Long(tema), new Long(id), new AsyncCallback<Void>() {
			@Override
			public void onSuccess(Void result) {
				// TODO Auto-generated method stub
				processResponse(requestId, response);
			}
			
			@Override
			public void onFailure(Throwable caught) {
				SC.warn("Error al eliminar");
			}
		});
	}

	private void executeFetch(final String requestId, final DSResponse response, final DSRequest request) {

		final Integer temaId;
		Criteria crit = request.getCriteria();
		if (crit != null)
			temaId = crit.getAttributeAsInt("tema");
		else
			temaId = null;

		Log.debug("obteniendo pregutnas de tema " + temaId);

		if (temaId != null) {

			temaAsignaturaService.obtenerPreguntasTema(new Long(temaId), new AsyncCallback<List<Pregunta>>() {
				public void onSuccess(List<Pregunta> result) {
					if (result != null) {
						// limpiar grid y añadir preguntas al GRID
						Record[] preguntas = new Record[result.size()];
						for (int i = 0; i < result.size(); i++) {
							Log.debug("Nº preguntas: " + result.size());

							preguntas[i] = new ListGridRecord();
							preguntas[i].setAttribute("titulo", result.get(i).getTitulo());
							preguntas[i].setAttribute("tipoPreguntaNombre", result.get(i).getTipoPregunta().getNombre());
							preguntas[i].setAttribute("tipoPreguntaID", result.get(i).getTipoPregunta().getId().intValue());
							preguntas[i].setAttribute("tipoPreguntaIDOriginal", result.get(i).getTipoPregunta().getId().intValue());
							preguntas[i].setAttribute("tema", temaId);
							preguntas[i].setAttribute("id", result.get(i).getId().intValue());
							preguntas[i].setAttribute("version", result.get(i).getVersion().intValue());
							preguntas[i].setAttribute("enunciado", result.get(i).getEnunciado());
							preguntas[i].setAttribute("codigo", result.get(i).getCodigo());
							preguntas[i].setAttribute("dificultad", result.get(i).getDificultad());
							preguntas[i].setAttribute("palabrasClave", result.get(i).getPalabrasClave());
							preguntas[i].setAttribute("fechaCreacion", result.get(i).getFechaCreacion());
							preguntas[i].setAttribute("fechaModificacion", result.get(i).getFechaModificacion());
							
							// necesario para el componente de respuesta
							preguntas[i].setAttribute("tipoPreguntaCodigo", result.get(i).getTipoPregunta().getId());
							
							// para el listado de preguntas de una evaluación, es la clave primaria de evaluacionpreguntaDataSource
							preguntas[i].setAttribute("preguntaId", result.get(i).getId());

							// addData(preguntas[i]);
						}

						response.setData(preguntas);
						response.setTotalRows(preguntas.length);
						processResponse(requestId, response);
					} else
						Window.alert("sin resultados");
				};

				@Override
				public void onFailure(Throwable caught) {
					Log.debug("Error obteniendo preguntas");
				};
			});

		} else {
			response.setData(new Record[] {});
			response.setTotalRows(0);
			processResponse(requestId, response);
		}

	}
}
