package es.upv.client.ui.ds;

import java.util.List;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.smartgwt.client.data.DSRequest;
import com.smartgwt.client.data.DSResponse;
import com.smartgwt.client.data.DataSource;
import com.smartgwt.client.data.DataSourceField;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.data.fields.DataSourceIntegerField;
import com.smartgwt.client.data.fields.DataSourceTextField;
import com.smartgwt.client.types.DSDataFormat;
import com.smartgwt.client.types.DSProtocol;
import com.smartgwt.client.util.SC;
import com.smartgwt.client.widgets.grid.ListGridRecord;

import es.upv.shared.dto.TipoPregunta;
import es.upv.shared.services.PreguntasServiceAsync;

public class TipoPreguntaDatasource extends DataSource {

	private PreguntasServiceAsync preguntasService = PreguntasServiceAsync.Util.getInstance();

	public TipoPreguntaDatasource() {

		setDataProtocol(DSProtocol.CLIENTCUSTOM);
		setDataFormat(DSDataFormat.CUSTOM);
		setClientOnly(false);

		DataSourceField f = new DataSourceIntegerField("id");
		f.setPrimaryKey(true);
		f.setHidden(true);
		addField(f);

		DataSourceField fCodigo = new DataSourceTextField("codigo", "Código", 5, true);
		DataSourceField fNombre = new DataSourceTextField("nombre", "Nombre", 128, true);
		DataSourceField fDesc = new DataSourceTextField("descripcion", "Descripción", 2000, true);

		DataSourceField fVers = new DataSourceIntegerField("version");
		fVers.setHidden(true);

		fCodigo.setRequired(true);
		addField(fCodigo);
		fNombre.setRequired(true);
		addField(fNombre);
		addField(fDesc);
		addField(fVers);
	}

	@Override
	protected Object transformRequest(DSRequest request) {
		String requestId = request.getRequestId();
		DSResponse response = new DSResponse();
		response.setAttribute("clientContext", request.getAttributeAsObject("clientContext"));
		// Asume success
		response.setStatus(0);

		switch (request.getOperationType()) {
		case FETCH:
			executeFetch(requestId, response, request);
			break;
		case ADD:
			executeAdd(requestId, request, response);
			break;
		case UPDATE:
			executeUpdate(requestId, request, response);
			break;
		case REMOVE:
			executeRemove(requestId, response, request);
			break;
		default:
			// Operation not implemented.
			break;
		}
		return request.getData();
	}

	private void executeRemove(final String requestId, final DSResponse response, final DSRequest request) {
		Log.debug("execute remove");
		
		// obtener el tema
		Record r = new Record(request.getData());
		int id = r.getAttributeAsInt("id");
		
		preguntasService.eliminaTipoPregunta(new Long(id), new AsyncCallback<Void>() {
			@Override
			public void onSuccess(Void result) {
				// TODO Auto-generated method stub
				processResponse(requestId, response);
			}
			
			@Override
			public void onFailure(Throwable caught) {
				SC.warn("Error al eliminar");
			}
		});
	}

	private void executeFetch(final String requestId, final DSResponse response, final DSRequest request) {
		preguntasService.obtenerTiposPregunta(new AsyncCallback<List<TipoPregunta>>() {

			public void onSuccess(List<TipoPregunta> result) {
				if (result != null) {

					Record[] tipos = new Record[result.size()];
					for (int i = 0; i < result.size(); i++) {
						Log.debug("Nº tipos: " + result.size());

						tipos[i] = new ListGridRecord();
						tipos[i].setAttribute("id", result.get(i).getId().intValue());
						tipos[i].setAttribute("version", result.get(i).getVersion().intValue());
						tipos[i].setAttribute("codigo", result.get(i).getCodigo());
						tipos[i].setAttribute("nombre", result.get(i).getNombre());
						tipos[i].setAttribute("descripcion", result.get(i).getDescripcion());
					}

					response.setData(tipos);
					response.setTotalRows(tipos.length);
					processResponse(requestId, response);
				} else
					Log.debug("sin resultados obteniendo los tipos");
			};

			@Override
			public void onFailure(Throwable caught) {
				Log.debug("Error obteniendo tipos");
			};
		});

	}

	private void executeUpdate(final String requestId, final DSRequest request, final DSResponse response) {
		Log.debug("update data");

		// para los daos, añadir y actualizar es lo mismo
		executeAdd(requestId, request, response);
	}

	private void executeAdd(final String requestId, final DSRequest request, final DSResponse response) {
		Log.debug("add tipo pregunta");

		// obtener el tema
		Record r = new Record(request.getData());

		TipoPregunta p = new TipoPregunta();
		p.setNombre(r.getAttribute("nombre"));
		p.setDescripcion(r.getAttribute("descripcion"));
		p.setCodigo(r.getAttribute("codigo"));

		if (r.getAttribute("version") != null)
			p.setVersion(r.getAttributeAsInt("version"));

		if (r.getAttribute("id") != null)
			p.setId(Long.valueOf(r.getAttributeAsString("id")));

		// rellena datos de la pregunta guardada
		preguntasService.actualizaTipoPregunta(p, new AsyncCallback<TipoPregunta>() {
			public void onSuccess(TipoPregunta result) {
				// actualizar datos del listado
				Record r = new ListGridRecord();
				r.setAttribute("id", result.getId().intValue());
				r.setAttribute("version", result.getVersion().intValue());
				r.setAttribute("nombre", result.getNombre());
				r.setAttribute("descripcion", result.getDescripcion());
				r.setAttribute("codigo", result.getCodigo());

				response.setData(new Record[] { r });
				// response.setInvalidateCache(true);
				processResponse(requestId, response);
			};

			@Override
			public void onFailure(Throwable caught) {
				Window.alert("Error al guardar");
			}
		});

	}

}
