package es.upv.client.utils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.JsArray;
import com.google.gwt.http.client.Request;
import com.google.gwt.http.client.RequestBuilder;
import com.google.gwt.http.client.RequestCallback;
import com.google.gwt.http.client.RequestException;
import com.google.gwt.http.client.Response;
import com.google.gwt.json.client.JSONException;
import com.google.gwt.json.client.JSONObject;
import com.google.gwt.json.client.JSONString;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.util.SC;

import es.upv.shared.model.PersonaOverlay;
import es.upv.shared.model.AsignaturaOverlay;

public class PersonaServiceUtils {

	public static void guardarUsuario(TipoAcceso tipoAcceso, Record datosAlumno, final JsonRequestAction successAction) {

		// obtener los valores del form y enviarlo a guardar
		// test guardar
		String urlsave = GWT.getHostPageBaseURL() + "services/api/" + tipoAcceso.getTipo() + "/save.json";
		RequestBuilder builder2 = new RequestBuilder(RequestBuilder.POST, urlsave);
		builder2.setHeader("Content-Type", "application/json");
		builder2.setHeader("Pragma", "no-cache");

		try {
			JSONObject nuevoAlumno = new JSONObject(datosAlumno.getJsObj());

			builder2.sendRequest(nuevoAlumno.toString(), new RequestCallback() {

				@Override
				public void onResponseReceived(Request request, Response response) {
					PersonaOverlay alumnoGuardado = PersonaOverlay.fromJson(response.getText());

					// ejecutar la accion de correcto
					successAction.successAction(new Record(alumnoGuardado));
				}

				@Override
				public void onError(Request request, Throwable exception) {
					SC.warn("Error al guardar el alumno");
				}
			});
		} catch (RequestException e) {
			SC.warn("Error al guardar el alumno");
		}

	}

	public static void obtenerAsignaturasAlumno(Long alumno, final JsonRequestAction actions) {
		Date now = new Date();
		String url = GWT.getHostPageBaseURL() + "services/api/alumnos/" + alumno + "/asignaturas.json?now="
				+ now.getTime();

		RequestBuilder builder = new RequestBuilder(RequestBuilder.GET, url);
		builder.setHeader("Pragma", "no-cache");

		try {
			builder.sendRequest(null, new RequestCallback() {
				public void onError(Request request, Throwable exception) {

					SC.warn("Error al obtener las asignaturas");
				}

				public void onResponseReceived(Request request, Response response) {
					if (200 == response.getStatusCode()) {
						try {
							// parse the response text into JSON
							JsArray<AsignaturaOverlay> data = AsignaturaOverlay.arrayFromJson(response.getText());

							List<AsignaturaOverlay> asignaturas = new ArrayList<AsignaturaOverlay>();
							for (int i = 0; i < data.length(); i++) {
								asignaturas.add(data.get(i));

								// action for one record
								actions.actionRecord(data.get(i));
							}

							// all records action
							actions.resultsAction(asignaturas);

						} catch (JSONException e) {
							SC.warn("Error al obtener las asignaturas");
						}
					} else {
						SC.warn("Error al obtener las asignaturas");
					}
				}
			});
		} catch (RequestException e) {
			SC.warn("Error al obtener las asignaturas");
		}
	}

	public static void matriculaAsignatura(String alumno, AsignaturaOverlay asignatura,
			final JsonRequestAction successAction) {

		// obtener los valores del form y enviarlo a guardar
		// test guardar
		int id = asignatura.getId();
		Long asigId = new Long(id);

		if (asigId != null) {
			String urlsave = GWT.getHostPageBaseURL() + "services/api/alumnos/" + alumno + "/nuevaMatricula.json?asig="
					+ asigId;
			RequestBuilder builder2 = new RequestBuilder(RequestBuilder.POST, urlsave);
			builder2.setHeader("Content-Type", "application/json");
			builder2.setHeader("Pragma", "no-cache");

			try {
				builder2.sendRequest(null, new RequestCallback() {

					@Override
					public void onResponseReceived(Request request, Response response) {

						if (response.getStatusCode() == Response.SC_NOT_MODIFIED)
							SC.say("Ya está matriculado de esa asignatura");
						else
							// ejecutar la accion de correcto
							successAction.successAction(null);
					}

					@Override
					public void onError(Request request, Throwable exception) {
						SC.warn("Error al matricular al alumno");
					}
				});
			} catch (RequestException e) {
				SC.warn("Error al matricular al alumno");
			}

		}
	}
	
	public static void desMatriculaAsignatura (String alumno, AsignaturaOverlay asignatura,
			final JsonRequestAction successAction) {

		// obtener los valores del form y enviarlo a guardar
		// test guardar
		int id = asignatura.getId();
		Long asigId = new Long(id);

		if (asigId != null) {
			String urlsave = GWT.getHostPageBaseURL() + "services/api/alumnos/" + alumno + "/desmatricula.json?asig="
					+ asigId;
			RequestBuilder builder2 = new RequestBuilder(RequestBuilder.POST, urlsave);
			builder2.setHeader("Content-Type", "application/json");
			builder2.setHeader("Pragma", "no-cache");

			try {
				builder2.sendRequest(null, new RequestCallback() {

					@Override
					public void onResponseReceived(Request request, Response response) {

						if (response.getStatusCode() == Response.SC_NOT_MODIFIED)
							SC.say("No est� matriculado de esa asignatura");
						else
							// ejecutar la accion de correcto
							successAction.successAction();
					}

					@Override
					public void onError(Request request, Throwable exception) {
						SC.warn("Error al desmatricular al alumno");
					}
				});
			} catch (RequestException e) {
				SC.warn("Error al desmatricular al alumno");
			}

		}
	}

	public static void validarUsuario(TipoAcceso tipoAcceso, String usuario, String clave, final JsonRequestAction successAction) {

		// obtener los valores del form y enviarlo a guardar
		// test guardar
		String urlsave = GWT.getHostPageBaseURL() + "services/api/" + tipoAcceso.getTipo() + "/validarUsuario.json";
		RequestBuilder builder2 = new RequestBuilder(RequestBuilder.POST, urlsave);
		builder2.setHeader("Content-Type", "application/json");
		builder2.setHeader("Pragma", "no-cache");

		// creo un alumno con los valores de usuario y clave
		// enviar la clave cifrada

		// Md5Utils.getMd5Digest(clave.getBytes());

		JSONObject obj = new JSONObject();
		obj.put("usuario", new JSONString(usuario));
		obj.put("clave", new JSONString(clave));

		try {

			builder2.sendRequest(obj.toString(), new RequestCallback() {

				@Override
				public void onResponseReceived(Request request, Response response) {

					if (response.getText() != null) {

						PersonaOverlay alumnoValidado = PersonaOverlay.fromJson(response.getText());

						if (alumnoValidado != null && alumnoValidado.getUsuario() != null)

							// ejecutar la accion de correcto
							successAction.successAction(new Record(alumnoValidado));
						else {
							successAction.failureAction();
						}
					} else {
						successAction.failureAction();
					}
				}

				@Override
				public void onError(Request request, Throwable exception) {
					successAction.failureAction();
				}
			});
		} catch (RequestException e) {
			successAction.failureAction();
		}
	}
	
	

	public  static void borrarUsuario(TipoAcceso tipoAcceso, String userId, final JsonRequestAction successAction) {
		// obtener los valores del form y enviarlo a
		// guardar
		String urlremove = GWT.getHostPageBaseURL() + "services/api/" + tipoAcceso.getTipo() + "/borrar/" + userId + ".json";
		RequestBuilder builder2 = new RequestBuilder(RequestBuilder.POST, urlremove);
		builder2.setHeader("Content-Type", "application/json");
		builder2.setHeader("Pragma", "no-cache");

		try {
			builder2.sendRequest(null, new RequestCallback() {

				@Override
				public void onResponseReceived(Request request, Response response) {

					// ejecutar la accion de
					// correcto
					successAction.successAction();
				}

				@Override
				public void onError(Request request, Throwable exception) {
					SC.warn("Error al eliminar el usuario");
				}
			});
		} catch (RequestException e) {
			SC.warn("Error al eliminar el usuario");
		}
	}
}
