package es.upv.server.service.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response.Status;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.orm.ObjectRetrievalFailureException;
import org.springframework.security.authentication.encoding.PasswordEncoder;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import com.google.gwt.http.client.Response;

import es.upv.server.dao.AlumnoDao;
import es.upv.server.dao.GenericDao;
import es.upv.server.model.Usuario;
import es.upv.server.service.AlumnoManager;
import es.upv.server.service.AsignaturaManager;
import es.upv.shared.model.Alumno;
import es.upv.shared.model.Asignatura;
import es.upv.shared.model.Matricula;
import es.upv.shared.model.MatriculaPK;
import es.upv.shared.model.Persona;

@Service("alumnoManager")
public class AlumnoManagerImpl implements AlumnoManager {
	private static final Log log = LogFactory.getLog(AlumnoManagerImpl.class);

	AlumnoDao alumnoDao;
	GenericDao<Matricula, MatriculaPK> matriculaDao;

	@Autowired
	PasswordEncoder passwordEncoder;

	@Autowired
	AsignaturaManager asignaturaManager;

	public AlumnoDao getAlumnoDao() {
		return alumnoDao;
	}

	@Autowired
	public void setAlumnoDao(AlumnoDao alumnoDao) {
		this.alumnoDao = alumnoDao;
	}

	@Autowired
	public void setMatriculaDao(GenericDao<Matricula, MatriculaPK> matriculaDao) {
		this.matriculaDao = matriculaDao;
	}

	@Override
	public List<Alumno> list() {
		List<Alumno> list = alumnoDao.getAll();

		return list;
	}

	@Override
	public Alumno saveAlumno(Alumno alumno) {
		// cifrar la clave del alumno
		// byte[] clave = alumno.getClave().getBytes(); //
		// Md5Utils.getMd5Digest(alumno.getClave().getBytes());

		boolean cambiarClave = false;
		if (passwordEncoder != null) {
			if (alumno.getVersion() == null)
				cambiarClave = true;
			else {
				Map<String, Object> queryParams = new HashMap<String, Object>();
				queryParams.put("idalumno", alumno.getId());
				// obtener la pass del usuario en DB y ver si ha cambiado
				@SuppressWarnings("rawtypes")
				List list = alumnoDao.findByNamedQuery("obtenerPasswordUsuario", queryParams);

				if (list != null && !list.isEmpty()) {
					String pass = (String) list.get(0);

					if (pass == null || !pass.equals(alumno.getClave()))
						cambiarClave = true;
				} else {
					// no tiene pasS ?
					cambiarClave = true;
				}
			}
		} else 
			log.debug("No hay definidio un password encoder");

		if (cambiarClave) {
			log.debug("Cambiando clave de usuario");
		
			alumno.setClave(passwordEncoder.encodePassword(alumno.getClave(), null));
		} else {
			log.debug("Password no ha cambiado");
		}

		return alumnoDao.save(alumno);
	}

	@Override
	public List<Asignatura> asignaturasAlumno(Long id) {
		return asignaturaManager.obtenerAsignaturasAlumno(id);
	}

	@Override
	public void nuevaMatricula(Long alumnId, Long asignaturaId) {
		Matricula m = new Matricula();
		Alumno al = alumnoDao.get(alumnId);
		Asignatura as = asignaturaManager.get(asignaturaId);

		MatriculaPK pk = new MatriculaPK();
		pk.setAlumno(al);
		pk.setAsignatura(as);

		m.setPk(pk);

		if (matriculaDao.exists(pk))
			throw new WebApplicationException(Response.SC_NOT_MODIFIED);

		matriculaDao.save(m);
	}

	@Override
	public void desmatricula(Long alumnId, Long asignaturaId) {
		Alumno al = alumnoDao.get(alumnId);
		Asignatura as = asignaturaManager.get(asignaturaId);

		MatriculaPK pk = new MatriculaPK();
		pk.setAlumno(al);
		pk.setAsignatura(as);

		try {
			matriculaDao.get(pk);
			matriculaDao.remove(pk);
		} catch (ObjectRetrievalFailureException ex) {
			ex.printStackTrace();
		}
	}

	@Override
	public Alumno validarUsuario (Persona alumno) {

		if (alumno == null || alumno.getUsuario() == null)
			return null;

		try {
			Usuario result = (Usuario) alumnoDao.loadUserByUsername(alumno.getUsuario());
			
			String clave = alumno.getClave();
			// cifrar la clave para comparar
			if (clave != null && passwordEncoder != null)
				clave = passwordEncoder.encodePassword(clave, null); 
			
			// validar que las claves sean las mismas
			if (clave != null && clave.equals(result.getClave())) {
				
				return alumnoDao.get(result.getId());
				
			}

		} catch (UsernameNotFoundException ex) {
			throw new WebApplicationException(Status.NOT_FOUND);
		}

		return new Alumno();
	}

	@Override
	public void borrarAlumno(Long alumnoId) {
		alumnoDao.remove(alumnoId);
	}
}
