package es.upv.server.service.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response.Status;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.encoding.PasswordEncoder;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import es.upv.server.dao.AutorDao;
import es.upv.server.dao.GenericDao;
import es.upv.server.model.Usuario;
import es.upv.server.service.AsignaturaManager;
import es.upv.server.service.AutorManager;
import es.upv.shared.model.Autor;
import es.upv.shared.model.Matricula;
import es.upv.shared.model.MatriculaPK;
import es.upv.shared.model.Persona;

@Service("autorManager")
public class AutorManagerImpl implements AutorManager {
	private static final Log log = LogFactory.getLog(AutorManagerImpl.class);

	AutorDao autorDao;
	GenericDao<Matricula, MatriculaPK> matriculaDao;

	@Autowired
	PasswordEncoder passwordEncoder;

	@Autowired
	AsignaturaManager asignaturaManager;

	public AutorDao getAutorDao() {
		return autorDao;
	}

	@Autowired
	public void setAutorDao(AutorDao AutorDao) {
		this.autorDao = AutorDao;
	}

	@Autowired
	public void setMatriculaDao(GenericDao<Matricula, MatriculaPK> matriculaDao) {
		this.matriculaDao = matriculaDao;
	}

	@Override
	public List<Autor> list() {
		List<Autor> list = autorDao.getAll();

		return list;
	}

	@Override
	public Autor saveAutor(Autor autor) {
		// cifrar la clave del alumno
		// byte[] clave = alumno.getClave().getBytes(); //
		// Md5Utils.getMd5Digest(alumno.getClave().getBytes());

		boolean cambiarClave = false;
		if (passwordEncoder != null) {
			if (autor.getVersion() == null)
				cambiarClave = true;
			else {
				Map<String, Object> queryParams = new HashMap<String, Object>();
				queryParams.put("idautor", autor.getId());
				// obtener la pass del usuario en DB y ver si ha cambiado
				@SuppressWarnings("rawtypes")
				List list = autorDao.findByNamedQuery("obtenerPasswordAutor", queryParams);

				if (list != null && !list.isEmpty()) {
					String pass = (String) list.get(0);

					if (pass == null || !pass.equals(autor.getClave()))
						cambiarClave = true;
				} else {
					// no tiene pasS ?
					cambiarClave = true;
				}
			}
		} else
			log.debug("No hay definidio un password encoder");

		if (cambiarClave) {
			log.debug("Cambiando clave de usuario");

			autor.setClave(passwordEncoder.encodePassword(autor.getClave(), null));
		} else {
			log.debug("Password no ha cambiado");
		}

		return autorDao.save(autor);
	}

	@Override
	public Autor validarUsuario(Persona autor) {

		if (autor == null || autor.getUsuario() == null)
			return null;

		try {
			Usuario result = (Usuario) autorDao.loadUserByUsername(autor.getUsuario());

			String clave = autor.getClave();
			// cifrar la clave para comparar
			if (clave != null && passwordEncoder != null)
				clave = passwordEncoder.encodePassword(clave, null);

			// validar que las claves sean las mismas
			if (clave != null && clave.equals(result.getClave())) {

				return autorDao.get(result.getId());

			}

		} catch (UsernameNotFoundException ex) {
			throw new WebApplicationException(Status.NOT_FOUND);
		}

		return new Autor();
	}

	@Override
	public void borrarAutor(Long autorId) {
		autorDao.remove(autorId);
	}
}
