package es.upv.server.services;

import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import es.upv.server.dao.EmployeeDao;
import es.upv.shared.dto.Employee;
import es.upv.shared.services.EmployeeService;

@Service("employeeService")
public class EmployeeServiceImpl implements EmployeeService {
	
	public EmployeeDao employeeDao;

	@PostConstruct
	public void init() {
	}
	
	@PreDestroy
	public void destroy() {
	}

	public Employee findEmployee(long employeeId) {
		
		return employeeDao.get(employeeId);
		
	}
	
	@Transactional(propagation=Propagation.REQUIRED, rollbackFor=Exception.class)
	public void saveEmployee(long employeeId, String name, String surname, String jobDescription) throws Exception {
			
		Employee employeeDTO = employeeDao.get(employeeId);
		
		if(employeeDTO == null) {
			employeeDTO = new Employee(employeeId, name,surname, jobDescription);
			employeeDao.save(employeeDTO);
		}
		
	}
	
	@Transactional(propagation=Propagation.REQUIRED, rollbackFor=Exception.class)
	public void updateEmployee(long employeeId, String name, String surname, String jobDescription) throws Exception {
		
		Employee employeeDTO = employeeDao.get(employeeId);
		
		if(employeeDTO != null) {
			employeeDTO.setEmployeeName(name);
			employeeDTO.setEmployeeSurname(surname);
			employeeDTO.setJob(jobDescription);
		}

	}
	
	@Transactional(propagation=Propagation.REQUIRED, rollbackFor=Exception.class)
	public void deleteEmployee(long employeeId) throws Exception {
		
		Employee employeeDTO = employeeDao.get(employeeId);
		
		if(employeeDTO != null)
			employeeDao.remove(employeeDTO.getEmployeeId());

	}
	
	@Transactional(propagation=Propagation.REQUIRED, rollbackFor=Exception.class)
	public void saveOrUpdateEmployee(long employeeId, String name, String surname, String jobDescription) throws Exception {

		Employee employeeDTO = new Employee(employeeId, name,surname, jobDescription);
		
		employeeDao.save(employeeDTO);
		
	}
	
	@Transactional(propagation=Propagation.REQUIRED, rollbackFor=Exception.class)
	public List<Employee> getAll () throws Exception {

		return employeeDao.getAll();
		
	}

	public EmployeeDao getEmployeeDAO() {
		return employeeDao;
	}

	@Autowired
	public void setEmployeeDAO(EmployeeDao employeeDAO) {
		this.employeeDao = employeeDAO;
	}

}
