package es.upv.server.services;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import es.upv.client.security.web.EvalticsForbiddenException;
import es.upv.server.dao.GenericDao;
import es.upv.shared.dto.Evaluacion;
import es.upv.shared.dto.EvaluacionAlumno;
import es.upv.shared.dto.EvaluacionPregunta;
import es.upv.shared.dto.RespuestaAlumno;
import es.upv.shared.services.EvaluacionService;

@Service("evaluacionService")
public class EvaluacionServiceImpl implements EvaluacionService {
	private final Log log = LogFactory.getLog(EvaluacionServiceImpl.class);

	GenericDao<Evaluacion, Long> evaluacionDao;
	GenericDao<EvaluacionAlumno, Long> evaluacionAlumnoDao;
	GenericDao<EvaluacionPregunta, Long> evaluacionPreguntaDao;
	GenericDao<RespuestaAlumno, Long> respuestasAlumnoDao;

	public GenericDao<Evaluacion, Long> getEvaluacionDao() {
		return evaluacionDao;
	}

	@Autowired
	public void setEvaluacionDao(GenericDao<Evaluacion, Long> evaluacionDao) {
		this.evaluacionDao = evaluacionDao;
	}

	public GenericDao<EvaluacionAlumno, Long> getEvaluacionAlumnoDao() {
		return evaluacionAlumnoDao;
	}

	@Autowired
	public void setEvaluacionAlumnoDao(GenericDao<EvaluacionAlumno, Long> evaluacionAlumnoDao) {
		this.evaluacionAlumnoDao = evaluacionAlumnoDao;
	}
	
	public GenericDao<EvaluacionPregunta, Long> getEvaluacionPreguntaDao() {
		return evaluacionPreguntaDao;
	}

	@Autowired
	public void setEvaluacionPreguntaDao(GenericDao<EvaluacionPregunta, Long> evaluacionPreguntaDao) {
		this.evaluacionPreguntaDao = evaluacionPreguntaDao;
	}

	public GenericDao<RespuestaAlumno, Long> getRespuestasAlumnoDao() {
		return respuestasAlumnoDao;
	}

	@Autowired
	public void setRespuestasAlumnoDao(GenericDao<RespuestaAlumno, Long> respuestasAlumnoDao) {
		this.respuestasAlumnoDao = respuestasAlumnoDao;
	}

	/*
	 * @Override public List<Pregunta> obtenerPreguntasTema(long temaId) throws
	 * Exception { Map<String, Object> params = new HashMap<String, Object>();
	 * 
	 * params.put("tema", temaId);
	 * 
	 * List result = evaluacionDao.findByNamedQuery("obtenerPreguntasDeTema",
	 * params); return (List<Pregunta>) result; }
	 */

	@Override
	public Evaluacion guardarEvaluacion(Evaluacion evaluacion) throws Exception {
		return evaluacionDao.save(evaluacion);
	}

	@Override
	public void guardarPreguntasEvaluacion(Long evaluacionID, List<EvaluacionPregunta> preguntas) {
		Evaluacion ev = evaluacionDao.get(evaluacionID);
		
		for (EvaluacionPregunta ep : preguntas ) {
			ep.setEvaluacionID(ev.getId());

			evaluacionPreguntaDao.save(ep);
		}
	}

	@Override
	public void obtenerEvaluacionesPublicadasAsignatura(Long asignaturaID) {
		// TODO Auto-generated method stub

	}

	@Override
	@Secured(value={"ROLE_ALUMNO"})
	public List<Evaluacion> obtenerEvaluacionesPendientesAlumno(Long alumnoID) throws EvalticsForbiddenException {
		Map<String, Object> queryParams = new HashMap<String, Object>();
		queryParams.put("alumno", alumnoID);
		queryParams.put("fechaActual", new Date());
		List<Evaluacion> evaluaciones = evaluacionDao.findByNamedQuery("obtenerEvaluacionesPendientes", queryParams);
		// SecurityContextHolder.getContext()

		return evaluaciones;
	}

	@Override
	public EvaluacionAlumno obtenerEvaluacionAlumno(Long alumnoID, Long evaluacionID) {

		Map<String, Object> queryParams = new HashMap<String, Object>();
		queryParams.put("alumno", alumnoID);
		queryParams.put("evaluacion", evaluacionID);

		List<EvaluacionAlumno> evaluaciones = evaluacionAlumnoDao.findByNamedQuery("obtenerEvaluacionesAlumno",
				queryParams);
		if (evaluaciones != null && !evaluaciones.isEmpty())
			return evaluaciones.get(0);

		return null;
	}

	@Override
	public List<RespuestaAlumno> obtenerPreguntasEvaluacionAlumno(Long evaluacionAlumnoID) {

		Map<String, Object> queryParams = new HashMap<String, Object>();
		queryParams.put("evaluacion", evaluacionAlumnoID);

		List<RespuestaAlumno> respuestas = respuestasAlumnoDao.findByNamedQuery("obtenerRespuestasEvaluacionAlumno",
				queryParams);

		return respuestas;
	}

	@Override
	@Transactional(rollbackFor=Exception.class)
	public EvaluacionAlumno comenzarEvaluacion(Long alumnoID, Long evaluacionID) {
		
		// crear una nueva evaluacion del alumno
		Evaluacion ev = evaluacionDao.get(evaluacionID);
		
		EvaluacionAlumno evAlumno = new EvaluacionAlumno();
		evAlumno.setEvaluacionID(evaluacionID);
		evAlumno.setAlumnoID(alumnoID);
		evAlumno.setAsignaturaID(ev.getAsignaturaID());
		
		evAlumno.setFecha(Calendar.getInstance().getTime());
		evAlumno.setFinalizado(false);
		evAlumno.setNota(0F);
		evAlumno.setComentarios("");
		evAlumno.setCorregido(false);
		
		try {
			evAlumno = evaluacionAlumnoDao.save(evAlumno);
			
			log.debug("Guardada evaluacion : " + evAlumno.getId());
			
			// una vez creada la evaluación, obtener las preguntas de la evaluacion
			Map<String, Object> queryParams = new HashMap<String, Object>();
			queryParams.put("evaluacion", evAlumno.getEvaluacionID());

			List<EvaluacionPregunta> preguntas = evaluacionPreguntaDao.findByNamedQuery("obtenerPreguntasEvaluacion",
					queryParams);
			
			// generar la evaluacion, de momento copiarlas todas a respuestas
			for (EvaluacionPregunta p :  preguntas) {
				RespuestaAlumno respuesta = new RespuestaAlumno();
				
				respuesta.setEvaluacionAlumnoID(evAlumno.getId());
				respuesta.setNota(null);
				respuesta.setEvaluacionPreguntaID(p.getId());
				
				respuestasAlumnoDao.save(respuesta);
			}
			
			return evAlumno;
			
		} catch (Exception e) {
			log.debug("Error guardando evaluacion de alumno " + evAlumno.toString());
		}
		
		return null;
	}

	@Override
	public RespuestaAlumno guardaRespuesta(Long evaluacionAlumnoID, RespuestaAlumno respuesta) {
		log.debug("Guardando respuesta para evaluacion " + evaluacionAlumnoID + ", con valor " + respuesta.getVf());
		
		return respuestasAlumnoDao.save(respuesta);
	}

	@Override
	public List<EvaluacionAlumno> obtenerEvaluacionesRealizadasAsignatura(Long asignaturaID) {
		Map<String, Object> queryParams = new HashMap<String, Object>();
		queryParams.put("asignatura", asignaturaID);
		return evaluacionAlumnoDao.findByNamedQuery("obtenerEvaluacionesFinalizadasAsignatura", queryParams);
	}

	@Override
	public void finalizarEvaluacion(Long evaluacionID) {
		EvaluacionAlumno ea = evaluacionAlumnoDao.get(evaluacionID);
		
		ea.setFinalizado(true);
		
		evaluacionAlumnoDao.save(ea);
	}
}
