package es.upv.server.services;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import es.upv.server.dao.GenericDao;
import es.upv.shared.dto.Pregunta;
import es.upv.shared.dto.PreguntaTema;
import es.upv.shared.dto.PreguntaTemaPK;
import es.upv.shared.dto.TemaAsignatura;
import es.upv.shared.services.TemaAsignaturaService;

@Service("temaAsignaturaServicee")
public class TemaAsignaturaServiceImpl implements TemaAsignaturaService {
	private static Log log = LogFactory.getLog(TemaAsignaturaServiceImpl.class);

	GenericDao<TemaAsignatura, Long> temaAsignaturaDao;
	GenericDao<Pregunta, Long> preguntaDao;
	GenericDao<PreguntaTema, PreguntaTemaPK> preguntaTemaDao;

	public GenericDao<TemaAsignatura, Long> getTemaAsignaturaDao() {
		return temaAsignaturaDao;
	}

	@Autowired
	public void setAsignaturaDao(
			GenericDao<TemaAsignatura, Long> temaAsignaturaDao) {
		this.temaAsignaturaDao = temaAsignaturaDao;
	}
	
	public GenericDao<PreguntaTema, PreguntaTemaPK> getPreguntaTemaDao() {
		return preguntaTemaDao;
	}

	@Autowired
	public void setPreguntaTemaDao(
			GenericDao<PreguntaTema, PreguntaTemaPK> preguntaTemaDao) {
		this.preguntaTemaDao = preguntaTemaDao;
	}
	
	public GenericDao<Pregunta, Long> getPreguntaDao() {
		return preguntaDao;
	}

	@Autowired
	public void setPreguntaDao(
			GenericDao<Pregunta, Long> preguntaDao) {
		this.preguntaDao = preguntaDao;
	}

	@Override
	public TemaAsignatura get(long asignaturaId) {
		return temaAsignaturaDao.get(asignaturaId);
	}

	public List<TemaAsignatura> list() {
		List<TemaAsignatura> list = temaAsignaturaDao.getAll();

		return list;
	}

	@Override
	public TemaAsignatura saveTema(TemaAsignatura a) {
		return temaAsignaturaDao.save(a);
	}

	@Override
	public List<TemaAsignatura> obtenerTemasAsignatura(long asignaturaId) {
		Map<String, Object> params = new HashMap<String, Object>();

		params.put("asignatura", asignaturaId);

		List<TemaAsignatura> result = temaAsignaturaDao.findByNamedQuery(
				"obtenerTemasDeAsignatura", params);
		return (List<TemaAsignatura>) result;
	}

	@Override
	public void deleteTema(long temaId) throws Exception {
		temaAsignaturaDao.remove(temaId);
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<Pregunta> obtenerPreguntasTema(long temaId) throws Exception {
		Map<String, Object> params = new HashMap<String, Object>();

		params.put("tema", temaId);

		@SuppressWarnings("rawtypes")
		List result = temaAsignaturaDao.findByNamedQuery(
				"obtenerPreguntasDeTema", params);
		return (List<Pregunta>) result;
	}

	@Override
	@Transactional(rollbackFor=Exception.class)
	public Pregunta guardaPreguntaDeTema(long temaId, Pregunta pregunta)
			throws Exception {
		
		log.debug("guardaPReguntadeTEma : " + temaId);
		
		TemaAsignatura tema = temaAsignaturaDao.get(temaId);
		
		if (pregunta.getId() != null && preguntaDao.exists(pregunta.getId())) {
			// Existe
			return preguntaDao.save(pregunta);
		} else {
			Pregunta p2 = preguntaDao.save(pregunta);
			PreguntaTema pt = new PreguntaTema();
			pt.setTemaAsignatura(tema);
			pt.setPregunta(p2);
			
			return preguntaTemaDao.save(pt).getPregunta();
		}
	}

	@Override
	@Transactional(rollbackFor=Exception.class)
	public void eliminaPreguntaTema(long temaId, long preguntaid) {
		TemaAsignatura tema = temaAsignaturaDao.get(temaId);
		Pregunta p = preguntaDao.get(preguntaid);
		
		PreguntaTemaPK pk = new PreguntaTemaPK();
		pk.setPregunta(p);
		pk.setTemaAsignatura(tema);
		
		preguntaDao.remove(preguntaid);
		preguntaTemaDao.remove(pk);
	}
}
