package es.upv.shared.dto;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Version;
import javax.xml.bind.annotation.XmlRootElement;

import es.upv.shared.model.Asignatura;
import es.upv.shared.model.Autor;

@Entity
@Table(name = "evaluacion")
@NamedQueries(value={@NamedQuery(name="obtenerEvaluacionesPendientes", 
				  query="select ev from Evaluacion ev , Matricula m where ev.asignatura = m.pk.asignatura and m.pk.alumno.id = :alumno and ev.publicado = 1 and ev.fechaCierre >= :fechaActual")})
@XmlRootElement(name = "Evaluacion", namespace = "http://evaltics.upv.es")
public class Evaluacion implements java.io.Serializable {
	private static final long serialVersionUID = 8307635165461944200L;

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@Column(name = "id")
	private Long id;

	@ManyToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "asignatura_id", updatable = false, insertable = false)
	private Asignatura asignatura;
	
	@Column(name="asignatura_id")
	private Long asignaturaID;

	@ManyToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "autor_id", updatable = false, insertable = false)
	private Autor autor;
	
	@Column(name="autor_id")
	private Long autorID;

	@ManyToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "tema_id", updatable = false, insertable = false)
	private TemaAsignatura tema;
	
	@Column(name="tema_id")
	private Long temaID;

	@Column(name = "titulo", nullable = false, length = 128)
	private String titulo;

	@Temporal(TemporalType.DATE)
	@Column(name = "fecha_apertura")
	private Date fechaApertura;

	@Temporal(TemporalType.DATE)
	@Column(name = "fecha_cierre")
	private Date fechaCierre;
	
	@Column(name="n_preguntas")
	private Integer numeroPreguntas;

	@Column(name = "valor")
	private Float valor;

	@Column(name = "grado_confianza")
	private Boolean gradoConfianza;

	@Column(name = "publicado")
	private Boolean publicado;

	@Version
	private Integer version;

	public Evaluacion() {
	}

	public Evaluacion(Long id) {
		this.id = id;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getTitulo() {
		return titulo;
	}

	public void setTitulo(String titulo) {
		this.titulo = titulo;
	}

	public Asignatura getAsignatura() {
		return asignatura;
	}
	
	public Long getAsignaturaID () {
		return asignaturaID;
	}

	public void setAsignatura(Asignatura asignatura) {
		this.asignatura = asignatura;
	}
	
	public void setAsignaturaID (Long id) {
		this.asignaturaID = id;
	}

	public Autor getAutor() {
		return autor;
	}
	
	public Long getAutorID () {
		return autorID;
	}

	public void setAutor(Autor autor) {
		this.autor = autor;
	}
	
	public void setAutorID (Long id) {
		this.autorID = id;
	}

	public TemaAsignatura getTema() {
		return tema;
	}
	
	public Long getTemaID () {
		return temaID;
	}

	public void setTema(TemaAsignatura tema) {
		this.tema = tema;
	}
	
	public void setTemaID (Long id) {
		this.temaID = id;
	}

	public Date getFechaApertura() {
		return fechaApertura;
	}

	public void setFechaApertura(Date fechaApertura) {
		this.fechaApertura = fechaApertura;
	}

	public Date getFechaCierre() {
		return fechaCierre;
	}

	public void setFechaCierre(Date fechaCierre) {
		this.fechaCierre = fechaCierre;
	}

	public Integer getNumeroPreguntas() {
		return numeroPreguntas;
	}

	public void setNumeroPreguntas(Integer numeroPreguntas) {
		this.numeroPreguntas = numeroPreguntas;
	}

	public Float getValor() {
		return valor;
	}

	public void setValor(Float valor) {
		this.valor = valor;
	}

	public Boolean getGradoConfianza() {
		return gradoConfianza;
	}

	public void setGradoConfianza(Boolean gradoConfianza) {
		this.gradoConfianza = gradoConfianza;
	}

	public Boolean getPublicado() {
		return publicado;
	}

	public void setPublicado(Boolean publicado) {
		this.publicado = publicado;
	}

	public Integer getVersion() {
		return this.version;
	}

	public void setVersion(Integer v) {
		this.version = v;
	}

	@Override
	public String toString() {
		return "Evaluacion [asignaturaID=" + asignaturaID + ", temaID="
				+ temaID + ", fechaApertura=" + fechaApertura + ", publicado="
				+ publicado + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((asignaturaID == null) ? 0 : asignaturaID.hashCode());
		result = prime * result + ((autorID == null) ? 0 : autorID.hashCode());
		result = prime * result
				+ ((fechaApertura == null) ? 0 : fechaApertura.hashCode());
		result = prime * result + ((temaID == null) ? 0 : temaID.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Evaluacion other = (Evaluacion) obj;
		if (asignaturaID == null) {
			if (other.asignaturaID != null)
				return false;
		} else if (!asignaturaID.equals(other.asignaturaID))
			return false;
		if (autorID == null) {
			if (other.autorID != null)
				return false;
		} else if (!autorID.equals(other.autorID))
			return false;
		if (fechaApertura == null) {
			if (other.fechaApertura != null)
				return false;
		} else if (!fechaApertura.equals(other.fechaApertura))
			return false;
		if (temaID == null) {
			if (other.temaID != null)
				return false;
		} else if (!temaID.equals(other.temaID))
			return false;
		return true;
	}
	
	

}