package es.upv.shared.model;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.Version;
import javax.xml.bind.annotation.XmlRootElement;

import org.codehaus.jackson.annotate.JsonIgnoreProperties;
import org.hibernate.annotations.Cascade;

@Entity
@Table(name = "asignatura")
@XmlRootElement(name = "Asignatura", namespace = "http://evaltics.upv.es")
@JsonIgnoreProperties(ignoreUnknown = true)
public class Asignatura implements java.io.Serializable {
	private static final long serialVersionUID = 6250021960097784447L;

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@Column(name = "id")
	private Long id;

	@Column(name = "nombre", nullable = false, length = 128)
	private String nombre;

	@Column(name = "codigo", nullable = false, length = 16)
	private String codigo;

	@Column(name = "siglas", nullable = false, length = 48)
	private String siglas;

	@Column(name = "descripcion")
	private String descripcion;

	@Column(name = "creditos_teoria")
	private Float credTeoria;

	@Column(name = "creditos_laboratorio")
	private Float credLaboratorio;

	@ManyToOne(fetch = FetchType.EAGER)
	@Cascade (value = { org.hibernate.annotations.CascadeType.SAVE_UPDATE})
	@JoinColumn(name = "curso_id", insertable = false, updatable = false)
	private Curso curso;
	
	@Column(name="curso_id")
	private Long cursoId;

	@ManyToOne(fetch = FetchType.EAGER)
	@Cascade (value = { org.hibernate.annotations.CascadeType.SAVE_UPDATE})
	@JoinColumn(name = "centro_id", insertable = false, updatable = false)
	private Centro centro;
	
	@Column(name="centro_id")
	private Long centroId;

	@Version
	private Integer version;

	public Asignatura() {
	}

	public Asignatura(Long id) {
		this.id = id;
	}

	public Asignatura(Long id, String nombre, String siglas, String descripcion) {
		this.id = id;
		this.nombre = nombre;
		this.siglas = siglas;
		this.descripcion = descripcion;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getNombre() {
		return nombre;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	public String getCodigo() {
		return codigo;
	}

	public void setCodigo(String cod) {
		this.codigo = cod;
	}

	public String getSiglas() {
		return siglas;
	}

	public void setSiglas(String siglas) {
		this.siglas = siglas;
	}

	public String getDescripcion() {
		return descripcion;
	}

	public void setDescripcion(String descripcion) {
		this.descripcion = descripcion;
	}

	public Float getCredTeoria() {
		return credTeoria;
	}

	public void setCredTeoria(Float credTeoria) {
		this.credTeoria = credTeoria;
	}

	public Float getCredLaboratorio() {
		return credLaboratorio;
	}

	public void setCredLaboratorio(Float credLaboratorio) {
		this.credLaboratorio = credLaboratorio;
	}

	public Curso getCurso() {
		return curso;
	}

	public void setCurso(Curso curso) {
		this.curso = curso;
	}

	@Transient
	public String getCursoNombre() {
		return this.curso != null ? this.curso.getNombre() : null;
	}

	public Long getCursoId() {
		return this.cursoId;
	}
	
	public void setCursoId (Long id) {
		this.cursoId = id;
	}

	public Centro getCentro() {
		return centro;
	}

	public Long getCentroId() {
		return this.centroId;
	}
	
	public void setCentroId (Long id) {
		this.centroId = id;
	}

	@Transient
	public String getCentroNombre() {
		return this.centro != null ? this.centro.getNombre() : null;
	}

	public void setCentro(Centro centro) {
		this.centro = centro;
	}

	public Integer getVersion() {
		return this.version;
	}

	public void setVersion(Integer v) {
		this.version = v;
	}

	@Override
	public String toString() {
		return "Centro [id=" + id + ", nombre=" + nombre + ", siglas=" + siglas + ", descripcion=" + descripcion + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((centro == null) ? 0 : centro.hashCode());
		result = prime * result + ((codigo == null) ? 0 : codigo.hashCode());
		result = prime * result + ((curso == null) ? 0 : curso.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Asignatura other = (Asignatura) obj;
		if (centro == null) {
			if (other.centro != null)
				return false;
		} else if (!centro.equals(other.centro))
			return false;
		if (codigo == null) {
			if (other.codigo != null)
				return false;
		} else if (!codigo.equals(other.codigo))
			return false;
		if (curso == null) {
			if (other.curso != null)
				return false;
		} else if (!curso.equals(other.curso))
			return false;
		return true;
	}

}